/*
 * $COPYRIGHT$
 * $Id: AboutBox.java,v 1.14 2002/05/22 19:35:37 borzy Exp $
 *
 * Date        Author               Changes
 * Apr 18 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.io.IOException;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.ActionListener;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;
import javax.swing.SwingConstants;
import org.media.hyperpad.gui.*;
import org.media.hyperpad.conf.ConfigHandler;
/**
 * Class meant to construct the AboutBox for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.14 $ $Date: 2002/05/22 19:35:37 $
 * @see String
 */  
public class AboutBox extends JDialog {
    static String name = "HyperPad";
    static String vers = "0.6a";
    static ConfigHandler ch;

    public AboutBox(JFrame f, String XMLpath) {
        super(f, "", true);
        this.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent ke) {
                if(ke.getKeyCode() == KeyEvent.VK_ESCAPE){
                    close();
                }
            }
            public void keyReleased(KeyEvent ke) {}
            public void keyTyped(KeyEvent ke) {}
        });
        ch = new ConfigHandler(XMLpath);
        JPanel container = new JPanel(new BorderLayout());
        JPanel name_ver = new JPanel(new BorderLayout());
        HypLabel nvl = new HypLabel(name + " v" + vers, 
                                MyResources.getImageIcon("aboutBoxImage"),
                                SwingConstants.LEFT);
        nvl.setFont(ch.getMenuFont());
        nvl.setIconTextGap(10);
        name_ver.add(nvl, "West");
        container.add(name_ver, BorderLayout.NORTH);
        final HypTabbedPane aboutPane = createAboutBox();
        aboutPane.setBorder(BorderFactory.createEmptyBorder(6,0,6,0));
        aboutPane.addMouseMotionListener(new java.awt.event.MouseMotionListener() {
            public void mouseDragged(java.awt.event.MouseEvent me) {}     
            public void mouseMoved(java.awt.event.MouseEvent me) {
                int tabIndex = aboutPane.getUI().tabForCoordinate(aboutPane, me.getX(), me.getY());
                HyperPanel.setTabForegroundAt(aboutPane, tabIndex);
            }
        });
        container.add(aboutPane, BorderLayout.CENTER);
        JPanel buttonPanel = new JPanel(new BorderLayout());
        HypButton button = new HypButton(MyResources.getTextValue("aboutBoxButton"));
        button.setFont(ch.getMenuFont());
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                close();
            }
        });
        buttonPanel.add(button, "East");
        container.add(buttonPanel, BorderLayout.SOUTH);
        container.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        getContentPane().add(container);
        setResizable(false);
        setTitle("About " + name);
        setModal(true);
        setSize(500,400);
        setLocationRelativeTo(f);
    }

    private static HypTabbedPane createAboutBox() {
        HypTabbedPane tp = new HypTabbedPane();
        tp.setFont(ch.getMenuFont());
        String[] tabKeys = HyperPanel.tokenize(HyperPanel.getResourceString("aboutBoxTabs"));
        for (int i = 0; i < tabKeys.length; i++) {
            JScrollPane sp = createTabs(tabKeys[i]);
            if (sp != null) {
                tp.addTab(MyResources.getTextValue(tabKeys[i] + "TabLabel"), sp);
            }
        }
        return tp;
    }

    private static JScrollPane createTabs(String key) {
        JScrollPane sp = new JScrollPane();
        HypTextPane tp = new HypTextPane(HypTextPane.NO_LINE_WRAP);
        try {
            tp.setPage(ClassLoader.getSystemResource(MyResources.getTextValue(key + "HTML")));
        }
        catch (IOException ioe) {
            ioe.printStackTrace();
        }
        tp.setEditable(false);
        tp.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        sp.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        sp.getViewport().add(tp);
        return sp;
    }

    private void close() {
        this.dispose();
    }
}
