/*
 * $COPYRIGHT$
 * $Id: BookmarkEditor.java,v 1.1 2002/01/23 17:24:34 borzy Exp $
 *
 * Date        Author               Changes
 * Jan 23 2002 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.util.Hashtable;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import javax.swing.Box;
import javax.swing.JTree;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JToolBar;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.filechooser.FileSystemView;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.media.hyperpad.gui.HypLabel;
import org.media.hyperpad.gui.HypButton;
import org.media.hyperpad.gui.HypTextField;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

/**
 * Class meant to construct the bookmark editor for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/23 17:24:34 $
 * @see String
 */  
public class BookmarkEditor extends JDialog {
    private JTree tree;
    private HypTextField name, url;
    private HypLabel urlLabel;
    private Hashtable urlValues = new Hashtable();
    private String oldName, oldUrl;
    private Element root, r;
    private Document document, doc;
    javax.swing.Timer timer = new javax.swing.Timer(300, null);

    public BookmarkEditor(JFrame f) {
        super(f, "", true);
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                new java.io.File(FileSystemView.getFileSystemView().getHomeDirectory().toString() +
                                 "/_HyperPadBookmarks.tmp").delete();
                HyperPanel.bmh.init();
                HyperPanel.MenuBar.remove(HyperPanel.MenuBar.getMenuCount()-2);
                HyperPanel.MenuBar.add(HyperPanel.bmh.createBookmarksMenu(), HyperPanel.MenuBar.getMenuCount()-1);
                HyperPanel.MenuBar.updateUI();
            }
        });
        init();
        JPanel main = new JPanel(new BorderLayout());
        main.add(createBMEToolbar(), BorderLayout.NORTH);

        tree = new JTree(createTree(root));
        tree.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
        tree.putClientProperty("JTree.lineStyle", "Angled");
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent tse) {
                try {
                    Object sel = tse.getNewLeadSelectionPath().getLastPathComponent();
                    name.setText(oldName = sel.toString());
                    url.setText(oldUrl = (String)urlValues.get(sel.toString()));
                }
                catch (Exception ex) {
                    name.setText(oldName = "");
                    url.setText(oldUrl = "");
                }
                finally {
                    if (url.getText().equals("")) {
                        url.setEnabled(false);
                        urlLabel.setEnabled(false);
                    }
                    else {
                        url.setEnabled(true);
                        urlLabel.setEnabled(true);
                    }
                }
            }
        });

        JScrollPane pane = new JScrollPane(tree);
        pane.setBorder(BorderFactory.createEmptyBorder(3,3,0,3));
        main.add(pane, BorderLayout.CENTER);

        JPanel labelPanel = new JPanel(new GridLayout(2, 1));
        labelPanel.add(new HypLabel("Name:"));
        labelPanel.add(urlLabel = new HypLabel("URL:"));
        JPanel TFPanel = new JPanel(new GridLayout(2, 1));
        name = new HypTextField(20);
        name.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent ke) {
                updateValues();
            }
        });
        url = new HypTextField(20);
        url.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent ke) {
                updateValues();
            }
        });
        TFPanel.add(name);
        TFPanel.add(url);
        
        JPanel editPane = new JPanel(new BorderLayout());
        editPane.add(labelPanel, BorderLayout.WEST);
        editPane.add(TFPanel, BorderLayout.CENTER);
        editPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        main.add(editPane, BorderLayout.SOUTH);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(main, BorderLayout.CENTER);

        setResizable(true);
        setTitle(HyperPanel.getResourceString("BookmarkEditorWindowLabel"));
        setModal(false);
        setSize(400,400);
        setLocationRelativeTo(f);
    }

    private void init() {
        this.root = HyperPanel.bmh.getRootElement();
        this.document = HyperPanel.bmh.getDocument();
    }

    private DefaultMutableTreeNode createTree(Node root) {
        String title = root.getChildNodes().item(1).getFirstChild().getNodeValue();
        DefaultMutableTreeNode cNode = new DefaultMutableTreeNode(title);
        NodeList childs = root.getChildNodes();
        for (int i = 0; i < childs.getLength(); i++) {
            if (childs.item(i).getParentNode().equals(root)) {
                if (childs.item(i).getNodeName().equalsIgnoreCase("bookmark")) {
                    String bookTitle = childs.item(i).getChildNodes().item(1).getFirstChild().getNodeValue();
                    String url = childs.item(i).getAttributes().getNamedItem("href").getNodeValue();
                    cNode.add(createTreeNode(bookTitle, url));
                }
                else if (childs.item(i).getNodeName().equalsIgnoreCase("folder")) {
                    cNode.add(createTree(childs.item(i)));
                }
            }
        }
        return cNode;
    }

    private void updateValues() {
        if (!timer.isRunning()) {
            timer.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    if (name.getText().length() == 0) {
                        name.setText(" ");
                    }
                    if (url.getText().length() == 0) {
                        url.setText(" ");
                    }
                    if (name.getText().trim().length() > 1 && name.getText().startsWith(" ")) {
                        name.setText(name.getText().trim());
                    }
                    if (url.getText().trim().length() > 1 && url.getText().startsWith(" ")) {
                        url.setText(url.getText().trim());
                    }
                    try {
                        setBookmarkNode(getBookmarkNode(oldName), name.getText(), url.getText());
                    } catch (Exception ex) {
                        setFolderNode(getFolderNode(oldName), name.getText());
                    }
                    urlValues.remove(oldName);
                    oldUrl = url.getText();
                    if (!oldName.equals(name.getText())) {
                        oldName = name.getText();
                        urlValues.put(oldName, oldUrl);
                        updateTree();
                    }
                    else {
                        urlValues.put(oldName, oldUrl);
                    }
                    timer.stop();
                }
            });
            timer.start();
        }
    }

    private void updateTree() {
        TreePath path = tree.getSelectionPath();
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
        node.setUserObject(oldName);
        tree.updateUI();
        tree.setSelectionPath(path);
    }

    private DefaultMutableTreeNode createTreeNode(String title, String url) {
        urlValues.put(title, url);
        return new DefaultMutableTreeNode(title);
    }

    private JToolBar createBMEToolbar() {
        JToolBar tb = new JToolBar();
        tb.setLayout(new BorderLayout());
        tb.setFloatable(false);
        JPanel buttonsPanel = new JPanel();
        String[] toolKeys = HyperPanel.tokenize(toolbar);
        for (int i = 0; i < toolKeys.length; i++) {
            if (toolKeys[i].equals("-")) {
                buttonsPanel.add(Box.createHorizontalStrut(5));
            } else {
                buttonsPanel.add(createToolbarButton(toolKeys[i]));
            }
        }
        buttonsPanel.add(Box.createHorizontalGlue());
        tb.add(buttonsPanel, BorderLayout.WEST);
        return tb;
    }

    private HypButton createToolbarButton(String key) {
        HypButton b = new HypButton(MyResources.getImageIcon(key + "Image")) {
            public float getAlignmentY() { return 0.5f; }
        };
        b.setRequestFocusEnabled(false);
        b.setMargin(new Insets(1,1,1,1));
        String tip = MyResources.getTextValue(key + "Tooltip");
        if (tip != null) {
            b.setToolTipText(tip);
        }
        if (key.equals("newBookmark")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    createNewBookmark("");
                }
            });        
        }
        else if (key.equals("newFolder")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    createNewFolder();
                }
            });        
        }
        else if (key.equals("up")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    moveUpSelected();
                }
            });        
        }
        else if (key.equals("down")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    moveDownSelected();
                }
            });        
        }
        else if (key.equals("expand")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int row = 0;
                    while (row < tree.getRowCount()) {
                        tree.expandRow(row);
                        row++;
                    }
                }
            });
        }
        else if (key.equals("collapse")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int row = tree.getRowCount();
                    while (row > 0) {
                        tree.collapseRow(row);
                        row--;
                    }
                }
            });
        }
        else if (key.equals("delete")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeSelectedNode();
                }
            });
        }
        else if (key.equals("save")) {
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    save(HyperPanel.bmh.getXMLPath());
                }
            });
        }
        return b;
    }

    private void moveUpSelected() {
        if (!tree.getSelectionPath().getLastPathComponent().equals(tree.getModel().getRoot())) {
            TreePath path = tree.getSelectionPath();
            DefaultMutableTreeNode selNode = (DefaultMutableTreeNode)path.getLastPathComponent();
            DefaultMutableTreeNode parent = (DefaultMutableTreeNode)selNode.getParent();
            int index = parent.getIndex(selNode);
            if (index > 0) {
                parent.remove(selNode);
                if (getBookmarkNode(parent.getChildAt(index-1).toString()) != null) {
                    parent.insert(selNode, index-1);
                }
                else if (getFolderNode(parent.getChildAt(index-1).toString()) != null) {
                    ((DefaultMutableTreeNode)parent.getChildAt(index-1)).add(selNode);
                    Object[] list = path.getPath();
                    java.util.Vector pathList = new java.util.Vector();
                    for (int i=0; i < list.length-1; i++) {
                        pathList.add(list[i]);
                    }
                    pathList.add(parent.getChildAt(index-1));
                    pathList.add(selNode);
                    path = new TreePath(pathList.toArray());
                }
                tree.updateUI();
                tree.setSelectionPath(path);
            }
            else if (index == 0) {
                if (!parent.equals(tree.getModel().getRoot())) {
                    Object[] list = path.getPath();
                    java.util.Vector pathList = new java.util.Vector();
                    for (int i=0; i < list.length-2; i++) {
                        pathList.add(list[i]);
                    }
                    pathList.add(selNode);
                    path = new TreePath(pathList.toArray());
                    int parIndex = parent.getParent().getIndex(parent);
                    parent.remove(selNode);
                    ((DefaultMutableTreeNode)parent.getParent()).insert(selNode, parIndex);
                    tree.updateUI();
                    tree.setSelectionPath(path);
                }
            }
        }
    }

    private void moveDownSelected() {
        if (!tree.getSelectionPath().getLastPathComponent().equals(tree.getModel().getRoot())) {
            TreePath path = tree.getSelectionPath();
            DefaultMutableTreeNode selNode = (DefaultMutableTreeNode)path.getLastPathComponent();
            DefaultMutableTreeNode parent = (DefaultMutableTreeNode)selNode.getParent();
            int index = parent.getIndex(selNode);
            if (index+1 < parent.getChildCount()) {
                parent.remove(selNode);
                if (getBookmarkNode(parent.getChildAt(index).toString()) != null) {
                    parent.insert(selNode, index+1);
                }
                else if (getFolderNode(parent.getChildAt(index).toString()) != null) {
                    ((DefaultMutableTreeNode)parent.getChildAt(index)).insert(selNode, 0);
                    Object[] list = path.getPath();
                    java.util.Vector pathList = new java.util.Vector();
                    for (int i=0; i < list.length-1; i++) {
                        pathList.add(list[i]);
                    }
                    pathList.add(parent.getChildAt(index));
                    pathList.add(selNode);
                    path = new TreePath(pathList.toArray());
                }
                tree.updateUI();
                tree.setSelectionPath(path);
            }
            else if (index+1 == parent.getChildCount()) {
                if (!parent.equals(tree.getModel().getRoot())) {
                    Object[] list = path.getPath();
                    java.util.Vector pathList = new java.util.Vector();
                    for (int i=0; i < list.length-2; i++) {
                        pathList.add(list[i]);
                    }
                    pathList.add(selNode);
                    path = new TreePath(pathList.toArray());
                    parent.remove(selNode);
                    ((DefaultMutableTreeNode)parent.getParent()).insert(selNode, parent.getParent().getIndex(parent)+1);
                    tree.updateUI();
                    tree.setSelectionPath(path);
                }
            }
        }
    }

    private Element getBookmarkNode(String name) {
        Element node = null;
        if (!name.equals("")) {
            NodeList childs = root.getElementsByTagName("bookmark");
            for (int i=0; i < childs.getLength(); i++) {
                if (childs.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().equals(name)) {
                    node = (Element)childs.item(i);
                }
            }
        }
        return node;
    }

    private void setBookmarkNode(Element node, String name, String url) {
        node.getChildNodes().item(1).getFirstChild().setNodeValue(name);
        node.removeAttribute("href");
        node.setAttribute("href", url);
    }

    private Element getFolderNode(String name) {
        Element node = null;
        if (!name.equals("")) {
            NodeList childs = root.getElementsByTagName("folder");
            for (int i=0; i < childs.getLength(); i++) {
                if (childs.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().equals(name)) {
                    node = (Element)childs.item(i);
                }
            }
        }
        return node;
    }

    private void setFolderNode(Element node, String name) {
        node.getChildNodes().item(1).getFirstChild().setNodeValue(name);
    }

    private void createNewBookmark(String url) {
        String title = "";
        try {
            if (url.equals("")) {
                url = HyperOptionPane.showInputDialog(HyperPanel.frame, 
                                                      HyperPanel.getResourceString("enterURLWindowTitle"),
                                                      HyperPanel.getResourceString("enterURLMessage"),
                                                      "");
            }
            if (url.equals("")) {
                return;
            }
            java.net.URL test = new java.net.URL(url);
            title = HyperOptionPane.showInputDialog(HyperPanel.frame, 
                                                    HyperPanel.getResourceString("enterNameWindowTitle"),
                                                    HyperPanel.getResourceString("enterNameMessage"),
                                                    url);
            if (title.length() > 0) {
                boolean exists = false;
                NodeList bookmarks = root.getElementsByTagName("bookmark");
                if (null != bookmarks) {
                    for (int i = 0; i < bookmarks.getLength(); i++) {
                        if (bookmarks.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().equals(title)) {
                            exists = true;
                        }
                    }
                }
                if (exists) {
                    HyperOptionPane.showDialog(HyperPanel.frame, 
                                               HyperPanel.getResourceString("bookmarkExistsWindowTitle"),
                                               HyperPanel.getResourceString("bookmarkExistsErrorMessage"),
                                               HyperOptionPane.DEFAULT_OPTION);
                    createNewBookmark(url);
                    title = "";
                }
            }
        } catch (Exception ex) {
            HyperOptionPane.showDialog(HyperPanel.frame, 
                                       HyperPanel.getResourceString("invalidURLWindowTitle"),
                                       HyperPanel.getResourceString("invalidURLErrorMessage"),
                                       HyperOptionPane.DEFAULT_OPTION);
            createNewBookmark("");
        }
        if (title.length() != 0 && url.length() != 0) {
            try {
                String folder = "";
                DefaultMutableTreeNode node =
                    (DefaultMutableTreeNode)tree.getSelectionPath().getLastPathComponent();
                if (getBookmarkNode(node.toString()) != null) {
                    node = (DefaultMutableTreeNode)node.getParent();
                }
                folder = node.toString();
                node.add(new DefaultMutableTreeNode(title));
                urlValues.put(title, url);
                NodeList folders = root.getElementsByTagName("folder");
                int j = 0;
                while (null != folders && j < folders.getLength() &&
                       !folders.item(j).getChildNodes().item(1).getFirstChild().getNodeValue().equals(folder)) {
                    j++;
                }
                folders.item(j).appendChild(document.createElement("bookmark"));
                ((Element)folders.item(j).getLastChild()).setAttribute("href", url);
                folders.item(j).getLastChild().appendChild(document.createElement("title"));
                folders.item(j).getLastChild().getFirstChild().appendChild(document.createTextNode(title));
            } catch (Exception ex) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getModel().getRoot();
                node.add(new DefaultMutableTreeNode(title));
                urlValues.put(title, url);
                root.appendChild(document.createElement("bookmark"));
                ((Element)root.getLastChild()).setAttribute("href", url);
                root.getLastChild().appendChild(document.createElement("title"));
                root.getLastChild().getFirstChild().appendChild(document.createTextNode(title));
            } finally {
                tree.updateUI();
                tmpSave();
            }
        }
    }

    private void createNewFolder() {
        String title = HyperOptionPane.showInputDialog(HyperPanel.frame,
                                                       HyperPanel.getResourceString("enterFolderNameWindowTitle"),
                                                       HyperPanel.getResourceString("enterFolderNameMessage"),
                                                       "");
        if (title.length() > 0) {
            boolean exists = false;
            NodeList folders = root.getElementsByTagName("folder");
            if (null != folders) {
                for (int i = 0; i < folders.getLength(); i++) {
                    if (folders.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().equals(title)) {
                        exists = true;
                    }
                }
            }
            if (exists) {
                HyperOptionPane.showDialog(HyperPanel.frame, 
                                           HyperPanel.getResourceString("folderExistsWindowTitle"),
                                           HyperPanel.getResourceString("folderExistsErrorMessage"),
                                           HyperOptionPane.DEFAULT_OPTION);
                createNewFolder();
                title = "";
            }
            else if (title.length() != 0) {
                try {
                    String folder = "";
                    DefaultMutableTreeNode node =
                        (DefaultMutableTreeNode)tree.getSelectionPath().getLastPathComponent();
                    if (getBookmarkNode(node.toString()) != null) {
                        node = (DefaultMutableTreeNode)node.getParent();
                    }
                    folder = node.toString();
                    int j = 0;
                    while (j < folders.getLength() &&
                           !folders.item(j).getChildNodes().item(1).getFirstChild().getNodeValue().equals(folder)) {
                        j++;
                    }
                    folders.item(j).appendChild(document.createElement("folder"));
                    folders.item(j).getLastChild().appendChild(document.createElement("title"));
                    folders.item(j).getLastChild().getFirstChild().appendChild(document.createTextNode(title));
                    node.add(new DefaultMutableTreeNode(title));
                } catch (Exception ex) {
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getModel().getRoot();
                    node.add(new DefaultMutableTreeNode(title));
                    root.appendChild(document.createElement("folder"));
                    root.getLastChild().appendChild(document.createElement("title"));
                    root.getLastChild().getFirstChild().appendChild(document.createTextNode(title));
                } finally {
                    tree.updateUI();
                    tmpSave();
                }
            }
        }
    }

    private void removeSelectedNode() {
        DefaultMutableTreeNode node = 
            (DefaultMutableTreeNode)tree.getSelectionPath().getLastPathComponent();
        if (null != node) {
            try {
                urlValues.remove(node.toString());
                node.removeFromParent();
                tree.updateUI();
            } catch (Exception ex) {}
        }
    }

    private void save(String path) {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.newDocument();
            r = doc.createElement("xbel");
            r.setAttribute("version", "1.0");
            doc.appendChild(r);
            r.appendChild(doc.createElement("title"));
            r.getElementsByTagName("title").item(0).appendChild(doc.createTextNode("Bookmarks"));
            saveTree((DefaultMutableTreeNode)tree.getModel().getRoot(), r);
            PrintWriter ow = new PrintWriter(new OutputStreamWriter (new FileOutputStream(path)));
            org.apache.xml.serialize.OutputFormat outForm = 
                new org.apache.xml.serialize.OutputFormat(doc);
            org.apache.xml.serialize.XMLSerializer serializer = 
                new org.apache.xml.serialize.XMLSerializer(ow, outForm);
            outForm.setIndenting(true);
            serializer.serialize(r);
        } catch (Exception ex) {}
    }

    private void saveTree(DefaultMutableTreeNode parent, Element parentNode) {
        for (int i=0; i < parent.getChildCount(); i++) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)parent.getChildAt(i); 
            if (urlValues.get(node.toString()) != null) {
                Element elem = doc.createElement("bookmark");
                elem.setAttribute("href", (String)urlValues.get(node.toString()));
                elem.appendChild(doc.createElement("title"));
                elem.getFirstChild().appendChild(doc.createTextNode(node.toString()));
                parentNode.appendChild(elem);
            }
            else {
                Element pNode = doc.createElement("folder");
                pNode.appendChild(doc.createElement("title"));
                pNode.getFirstChild().appendChild(doc.createTextNode(node.toString()));
                parentNode.appendChild(pNode);
                saveTree(node, pNode);
            }
        }
    }

    private void tmpSave() {
        try {
            String path = FileSystemView.getFileSystemView().getHomeDirectory().toString() +
                "/_HyperPadBookmarks.tmp";
            PrintWriter outW = new PrintWriter(new OutputStreamWriter (new FileOutputStream(path)));
            org.apache.xml.serialize.OutputFormat outForm = 
                new org.apache.xml.serialize.OutputFormat(document);
            org.apache.xml.serialize.XMLSerializer serializer = 
                new org.apache.xml.serialize.XMLSerializer(outW, outForm);
            outForm.setIndenting(true);
            serializer.serialize(root);
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            InputStream ins = (InputStream)new FileInputStream(path);
            if (ins != null) {
                InputSource is = new InputSource(ins);
                DocumentBuilder builder = factory.newDocumentBuilder();
                document = builder.parse(is); 
                root = document.getDocumentElement();
            }
        } catch (Exception ex) {}
    }
    
    private final String toolbar = "newBookmark newFolder - up down - save - expand collapse - delete";
}
