/*
 * $COPYRIGHT$
 * $Id: BookmarksHandler.java,v 1.5 2002/03/25 17:57:54 neuro Exp $
 *
 * Date        Author               Changes
 * Dec 18 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.net.URL;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;
import org.media.hyperpad.conf.ConfigHandler;
import org.media.hyperpad.gui.HypMenu;
import org.media.hyperpad.gui.HypMenuItem;
/**
 * Class meant to handle the bookmarks for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.5 $ $Date: 2002/03/25 17:57:54 $
 * @see String
 */  
public class BookmarksHandler {
    private Document doc;
    private DocumentBuilderFactory factory;
    private Element e;
    private HypMenu menu;
        
    public BookmarksHandler() {
        init();
    }
    
    protected void init() {
        try {
            factory = DocumentBuilderFactory.newInstance();
            InputStream ins = getXML(getXMLPath());
            if (ins != null) {
                InputSource is = new InputSource(ins);
                DocumentBuilder builder = factory.newDocumentBuilder();
                doc = builder.parse(is); 
                e = doc.getDocumentElement();
            }
        }
        catch (Exception ex) {
            createXML(getXMLPath());
        }
    }

    public HypMenu createBookmarksMenu() {
        menu = createBookmarkMenu(e);
        return menu;
    }

    private InputStream getXML(String path) throws IOException {
        InputStream ins = (InputStream)new FileInputStream(path);
        if (ins != null) {
            return ins;
        }
        else {
            return null;
        }
    }

    public Element getRootElement() {
        return e;
    }

    public Document getDocument() {
        return doc;
    }

    public void save() {
        saveXML(getXMLPath());
    }

    protected String getXMLPath() {
        File f = new File( System.getProperty("user.home") + HyperPanel.bookmarksFileName);
        return f.toString();
    }


    private HypMenu createBookmarkMenu(Node root) {
        String title = root.getChildNodes().item(1).getFirstChild().getNodeValue();
        HypMenu cMenu = new HypMenu(title);
        if (title.equals("Bookmarks")) {
            char mnemonic = HyperPanel.getResourceMnemonic(title.toLowerCase() + "Label");
            if (mnemonic != '!') {
                cMenu.setMnemonic(mnemonic);
            }
        }
        cMenu.setFont(HyperPanel.ch.getMenuFont());
        int itemNR = 0;
        NodeList childs = root.getChildNodes();
        for (int i = 0; i < childs.getLength(); i++) {
            if (childs.item(i).getParentNode().equals(root)) {
                if (childs.item(i).getNodeName().equalsIgnoreCase("bookmark")) {
                    String bookTitle = childs.item(i).getChildNodes().item(1).getFirstChild().getNodeValue();
                    String url = childs.item(i).getAttributes().getNamedItem("href").getNodeValue();
                    cMenu.add(createMenuItem(bookTitle, url), itemNR++);
                }
                else if (childs.item(i).getNodeName().equalsIgnoreCase("folder")) {
                    cMenu.add(createBookmarkMenu(childs.item(i)), itemNR++);
                }
            }
        }
        if (root.equals(e)) {
            HypMenuItem edit = new HypMenuItem("Edit bookmarks...");
            edit.setFont(HyperPanel.ch.getMenuFont());
            edit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    init();
                    BookmarkEditor be = new BookmarkEditor(HyperPanel.frame);
                    be.setVisible(true);
                }
            });
            cMenu.add(edit, 0);
            cMenu.add(new javax.swing.JSeparator(), 1);
        }
        cMenu.addSeparator();
        cMenu.add(createFolderMenu(title));
        
        return cMenu;
    }
    
    private HypMenu createMenu(String title) {
        HypMenu hm = new HypMenu(title);
        hm.setFont(HyperPanel.ch.getMenuFont());
        hm.addSeparator();
        hm.add(createFolderMenu(title));
        return hm;
    }
    
    private HypMenuItem createMenuItem(String title, final String url) {
        HypMenuItem mi = new HypMenuItem(title);
        mi.setFont(HyperPanel.ch.getMenuFont());
        mi.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                OpeningThread oth = new OpeningThread(null, url);
                oth.run();
            }
        });
        return mi;
    }

    private HypMenu createFolderMenu(final String folderName) {
        HypMenu fldmenu = new HypMenu("Folder Actions");
        fldmenu.setFont(HyperPanel.ch.getMenuFont());
        HypMenuItem addBookmark = new HypMenuItem("Add bookmark here");
        addBookmark.setFont(HyperPanel.ch.getMenuFont());
        addBookmark.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                addBookmark(folderName);
            }
        });
        HypMenuItem addFolder = new HypMenuItem("Add folder here");
        addFolder.setFont(HyperPanel.ch.getMenuFont());
        addFolder.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                addFolder(folderName, true);
            }
        });
        fldmenu.add(addBookmark);
        fldmenu.add(addFolder);
        return fldmenu;
    }

    protected void addBookmark(String folder) {
        String url = "";
        if (HyperPanel._sourceDocPath.equals("Unknown")) {
            HyperOptionPane.showDialog(HyperPanel.frame, 
                                       "URL unknown",
                                       "You must save the script first and then bookmark it.",
                                       HyperOptionPane.DEFAULT_OPTION);
        }
        else if (HyperPanel._sourceDocPath.startsWith("Unknown")) {
            url = HyperPanel._sourceDocPath.substring(7);
        }
        else {
            url = "file:"+HyperPanel._sourceDocPath;
        }
        if (!url.equals("")) {
            try {
                URL test = new URL(url);
                String title = HyperOptionPane.showInputDialog(HyperPanel.frame, 
                                                               "Bookmark name:", "Enter a name for this URL:", url);
                if (title.length() > 0) {
                    boolean exists = false;
                    NodeList bookmarks = e.getElementsByTagName("bookmark");
                    if (null != bookmarks) {
                        for (int i = 0; i < bookmarks.getLength(); i++) {
                            if (bookmarks.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().equals(title)) {
                                exists = true;
                            }
                        }
                    }
                    if (exists) {
                        HyperOptionPane.showDialog(HyperPanel.frame, 
                                                   "Folder exists",
                                                   "A bookmark with this name is already exists.",
                                                   HyperOptionPane.DEFAULT_OPTION);
                        addBookmark(folder);
                    }
                    else if (folder.equals("Bookmarks") && title.length() > 0) {
                        menu.add(createMenuItem(title, url), getLastMenuItemPosition(-1, menu));
                        e.appendChild(doc.createElement("bookmark"));
                        ((Element)e.getLastChild()).setAttribute("href", url);
                        e.getLastChild().appendChild(doc.createElement("title"));
                        e.getLastChild().getFirstChild().appendChild(doc.createTextNode(title));
                        saveXML(getXMLPath());
                    }
                    else if (title.length() > 0) {
                        insertMenuItem(menu, folder, title, url);
                        NodeList folders = e.getElementsByTagName("folder");
                        int j = 0;
                        while (null != folders && j < folders.getLength() &&
                               !folders.item(j).getChildNodes().item(1).getFirstChild().getNodeValue().equals(folder)) {
                            j++;
                        }
                        folders.item(j).appendChild(doc.createElement("bookmark"));
                        ((Element)folders.item(j).getLastChild()).setAttribute("href", url);
                        folders.item(j).getLastChild().appendChild(doc.createElement("title"));
                        folders.item(j).getLastChild().getFirstChild().appendChild(doc.createTextNode(title));
                        saveXML(getXMLPath());
                    }
                }
            } catch (Exception ex) {
                HyperOptionPane.showDialog(HyperPanel.frame, 
                                           "Malformed URL",
                                           "Your source document path is not a valid URL.",
                                           HyperOptionPane.DEFAULT_OPTION);
            }
        }
    }

    protected void addFolder(String folder, boolean local) {
        String title = HyperOptionPane.showInputDialog(HyperPanel.frame, "Folder name:", "Enter folder name:", "");
        if (title.length() > 0) {
            boolean exists = false;
            NodeList folders = e.getElementsByTagName("folder");
            if (null != folders) {
                for (int i = 0; i < folders.getLength(); i++) {
                    if (folders.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().equals(title)) {
                        exists = true;
                    }
                }
            }
            if (exists) {
                HyperOptionPane.showDialog(HyperPanel.frame, 
                                           "Folder exists",
                                           "A folder with this name is already exists.",
                                           HyperOptionPane.DEFAULT_OPTION);
                addFolder(folder, local);
            }
            else {
                if (folder.equals("Bookmarks")) {
                    if (local) {
                        menu.add(createMenu(title), menu.getItemCount()-2);
                    }
                    e.appendChild(doc.createElement("folder"));
                    e.getLastChild().appendChild(doc.createElement("title"));
                    e.getLastChild().getFirstChild().appendChild(doc.createTextNode(title));
                    saveXML(getXMLPath());
                }
                else {
                    if (local) {
                        insertMenu(menu, folder, title);
                    }
                    int j = 0;
                    while (j < folders.getLength() &&
                           !folders.item(j).getChildNodes().item(1).getFirstChild().getNodeValue().equals(folder)) {
                        j++;
                    }
                    folders.item(j).appendChild(doc.createElement("folder"));
                    folders.item(j).getLastChild().appendChild(doc.createElement("title"));
                    folders.item(j).getLastChild().getFirstChild().appendChild(doc.createTextNode(title));
                    saveXML(getXMLPath());
                }
            }
        }
    }

    private void insertMenu(HypMenu root, String parent, String title) {
        for (int i = 0; i < root.getItemCount(); i++) {
            if (root.getItem(i) instanceof HypMenu) {
                if (root.getItem(i).getText().equals(parent)) {
                    root.getItem(i).add(createMenu(title), ((HypMenu)root.getItem(i)).getItemCount()-2);
                }
                else {
                    insertMenu((HypMenu)root.getItem(i), parent, title);
                }
            }
        }
    }

    private void insertMenuItem(HypMenu root, String parent, String title, String url) {
        for (int i = 0; i < root.getItemCount(); i++) {
            if (root.getItem(i) instanceof HypMenu) {
                if (root.getItem(i).getText().equals(parent)) {
                    root.getItem(i).add(createMenuItem(title, url), getLastMenuItemPosition(i, root));
                }
                else {
                    insertMenuItem((HypMenu)root.getItem(i), parent, title, url);
                }
            }
        }
    }

    private int getLastMenuItemPosition(int menuNR, HypMenu root) {
        int i = 0;
        if (menuNR >= 0) {
            root = (HypMenu)root.getItem(menuNR);
        }
        while (root.getMenuComponent(i) instanceof HypMenuItem) {
            i++;
        }
        return i;
    }

    private void createXML(String path) {
        try {
            factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.newDocument();
            e = doc.createElement("xbel");
            e.setAttribute("version", "1.0");
            doc.appendChild(e);
            e.appendChild(doc.createElement("title"));
            getElement("title", "").appendChild(doc.createTextNode("Bookmarks"));
            saveXML(path);
        }
        catch (Exception exc) {}
    }

    private Element getElement(String parent, String child) {
        NodeList nl = e.getElementsByTagName(parent);
        Element elem = (Element) nl.item(0);
        if (!child.equals("")) {
            NodeList nl2 = elem.getElementsByTagName(child);
            elem = (Element) nl2.item(0);
        }
        return elem;
    }

    private void saveXML(String path) {
        try {
            PrintWriter out = new PrintWriter(new OutputStreamWriter (new FileOutputStream(path)));
            org.apache.xml.serialize.OutputFormat outForm = 
                new org.apache.xml.serialize.OutputFormat(doc);
            org.apache.xml.serialize.XMLSerializer serializer = 
                new org.apache.xml.serialize.XMLSerializer(out, outForm);
            outForm.setIndenting(true);
            serializer.serialize(e);
            init();
        }
        catch (Exception exc) {}
    }
}
