/*
 * $COPYRIGHT$
 * $Id: FindBox.java,v 1.19 2001/11/13 10:27:34 borzy Exp $
 *
 * Date        Author               Changes
 * Jul 02 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import org.media.hyperpad.gui.*;
import org.media.hyperpad.conf.ConfigHandler;
/**
 * Class meant to construct the FindBox for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.19 $ $Date: 2001/11/13 10:27:34 $
 * @see String
 */  
public class FindBox extends JDialog {
    private ConfigHandler ch = new ConfigHandler(HyperPanel.XMLpath);
    private HypTabbedPane tabp = new HypTabbedPane();
    private HypTextField findInput, replaceFindInput, replaceInput, gotoInput;
    private HypButton findButton, replaceButton, gotoButton;
    private int start_offset = 0;
    private int end_offset = 0;
    private String searchFor = "";
    private JFrame frame;
    private boolean result = false;
    private boolean secondTimeNotFound = false;
    private HypCheckBox case_sensitive;
    private HypCheckBox search_backward;
    private HypTextPane source;
    
    public FindBox(JFrame f, HypTextPane tp, final int selectedTab) {
        super(f, "", true);
        this.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent ke) {
                if(ke.getKeyCode() == java.awt.event.KeyEvent.VK_ESCAPE){
                    close();
                }
            }
        });
        this.addFocusListener(new java.awt.event.FocusListener() {
            public void focusGained(java.awt.event.FocusEvent fe) {
                inicialize(selectedTab);
            }
            public void focusLost(java.awt.event.FocusEvent fe) {}
        });
        frame = f;
        source = tp;
        tabp.setFont(ch.getMenuFont());
        tabp.addTab("Find", findTab());
        tabp.addTab("Replace", replaceTab());
        if (HyperPanel.jTabPane.getSelectedIndex() == 1) {
            tabp.addTab("Go To", gotoTab());
        }
        tabp.setRequestFocusEnabled(false);
        tabp.setSelectedIndex(selectedTab);
        inicialize(selectedTab);
        tabp.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent me) {
                int index = tabp.getUI().tabForCoordinate(tabp, me.getX(), me.getY());
                inicialize(index);
            }
        });
        tabp.addMouseMotionListener(new java.awt.event.MouseMotionListener() {
            public void mouseDragged(java.awt.event.MouseEvent me) {}     
            public void mouseMoved(java.awt.event.MouseEvent me) {
                int tabIndex = tabp.getUI().tabForCoordinate(tabp, me.getX(), me.getY());
                HyperPanel.setTabForegroundAt(tabp, tabIndex);
            }
        });
        getContentPane().add(tabp);
        setTitle("HyperPad: "+HyperPanel.getResourceString("FRGBoxWindowTitle"));
        setModal(true);
        pack();
        setLocationRelativeTo(frame);
    }

    private JPanel findTab() {
        JPanel findPanel = new JPanel(new BorderLayout());
        JPanel main = new JPanel(new BorderLayout());
        HypLabel label = new HypLabel(HyperPanel.getResourceString("findtextfieldLabel"));
        label.setFont(ch.getMenuFont());
        label.setForeground(Color.black);
        findInput = new HypTextField();
        case_sensitive = new HypCheckBox(HyperPanel.getResourceString("case_sensitiveLabel"));
        case_sensitive.setFont(ch.getMenuFont());
        case_sensitive.setRequestFocusEnabled(false);
        search_backward = new HypCheckBox(HyperPanel.getResourceString("search_backwardLabel"));
        search_backward.setFont(ch.getMenuFont());
        search_backward.setRequestFocusEnabled(false);
        search_backward.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (search_backward.isSelected()) {
                    start_offset -= findInput.getText().length();
                }
                else if (!search_backward.isSelected()) {
                    start_offset += findInput.getText().length();
                }
            }
        });
        JPanel checkbox_panel = new JPanel(new FlowLayout(FlowLayout.CENTER,20,0));
        checkbox_panel.add(case_sensitive);
        checkbox_panel.add(search_backward);
        checkbox_panel.setBorder(BorderFactory.createEmptyBorder(12,0,0,0));
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(label, BorderLayout.WEST);
        panel.add(findInput, BorderLayout.CENTER);
        panel.setPreferredSize(new Dimension(10,24));
        main.add(panel, BorderLayout.NORTH);
        main.add(checkbox_panel, BorderLayout.SOUTH);
        main.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        JPanel button_panel = buttonPanel("find");
        button_panel.setBorder(BorderFactory.createEmptyBorder(0,6,6,6));
        findPanel.add(main, BorderLayout.CENTER);
        findPanel.add(button_panel, BorderLayout.SOUTH);
        return findPanel;
    }

    private JPanel replaceTab() {
        JPanel replacePanel = new JPanel(new BorderLayout());
        JPanel main = new JPanel(new BorderLayout());
        HypLabel findLabel = new HypLabel(HyperPanel.getResourceString("findtextfieldLabel"));
        findLabel.setFont(ch.getMenuFont());
        findLabel.setForeground(Color.black);
        HypLabel replaceLabel = new HypLabel(HyperPanel.getResourceString("replacetextfieldLabel"));
        replaceLabel.setFont(ch.getMenuFont());
        replaceLabel.setForeground(Color.black);
        replaceFindInput = new HypTextField();
        replaceFindInput.requestFocus();
        replaceInput = new HypTextField();
        case_sensitive = new HypCheckBox(HyperPanel.getResourceString("case_sensitiveLabel"));
        case_sensitive.setFont(ch.getMenuFont());
        case_sensitive.setRequestFocusEnabled(false);
        search_backward = new HypCheckBox(HyperPanel.getResourceString("search_backwardLabel"));
        search_backward.setFont(ch.getMenuFont());
        search_backward.setRequestFocusEnabled(false);
        search_backward.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (search_backward.isSelected()) {
                    start_offset -= replaceFindInput.getText().length();
                }
                else if (!search_backward.isSelected()) {
                    start_offset += replaceFindInput.getText().length();
                }
            }
        });
        JPanel checkbox_panel = new JPanel(new FlowLayout(FlowLayout.CENTER,20,0));
        checkbox_panel.add(case_sensitive);
        checkbox_panel.add(search_backward);
        checkbox_panel.setBorder(BorderFactory.createEmptyBorder(12,0,0,0));
        JPanel textfieldPanel = new JPanel(new GridLayout(2,1,0,5));
        textfieldPanel.add(replaceFindInput);
        textfieldPanel.add(replaceInput);
        textfieldPanel.setPreferredSize(new Dimension(10,53));
        JPanel labelPanel = new JPanel(new GridLayout(2,1,0,5));
        labelPanel.add(findLabel);
        labelPanel.add(replaceLabel);
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(labelPanel, BorderLayout.WEST);
        panel.add(textfieldPanel, BorderLayout.CENTER);
        main.add(panel, BorderLayout.NORTH);
        main.add(checkbox_panel, BorderLayout.SOUTH);
        main.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        JPanel button_panel = buttonPanel("replace");
        button_panel.setBorder(BorderFactory.createEmptyBorder(0,6,6,6));
        replacePanel.add(main, BorderLayout.CENTER);
        replacePanel.add(button_panel, BorderLayout.SOUTH);
        return replacePanel;
    }

    private JPanel gotoTab() {
        JPanel gotoPanel = new JPanel(new BorderLayout());
        JPanel main = new JPanel(new GridLayout(2,1,0,5));
        HypLabel gotoLabel = new HypLabel(HyperPanel.getResourceString("gototextfieldLabel"));
        gotoLabel.setFont(ch.getMenuFont());
        gotoLabel.setForeground(Color.black);
        gotoInput = new HypTextField();
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(gotoLabel, BorderLayout.WEST);
        panel.add(gotoInput, BorderLayout.CENTER);
        panel.setPreferredSize(new Dimension(10,24));
        HypLabel countLabel = new HypLabel("Enter line number between {1-" + countLineNumber() + "}.");
        countLabel.setFont(ch.getMenuFont());
        countLabel.setForeground(Color.black);
        main.add(countLabel); 
        main.add(panel);
        main.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        JPanel button_panel = buttonPanel("goto");
        button_panel.setBorder(BorderFactory.createEmptyBorder(0,6,6,6));
        gotoPanel.add(main, BorderLayout.NORTH);
        gotoPanel.add(button_panel, BorderLayout.SOUTH);
        return gotoPanel;
    }

    private JPanel buttonPanel(String key) {
        String[] buttonKeys = new String[0];
        if (key.equals("find")) {
            buttonKeys = HyperPanel.tokenize(findButtons);
        }
        else if (key.equals("replace")) {
            buttonKeys = HyperPanel.tokenize(replaceButtons);
        }
        else if (key.equals("goto")) {
            buttonKeys = HyperPanel.tokenize(gotoButtons);
        }
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.RIGHT,5,0));
        for (int i = 0; i < buttonKeys.length; i++) {
            HypButton button = new HypButton(MyResources.getTextValue(buttonKeys[i] + "ButtonLabel"));
            button.setFont(ch.getMenuFont()); 
            button.setRequestFocusEnabled(false);
            if (buttonKeys[i].equals("find")) {
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        find();
                    }
                });
                findButton = button;
                panel.add(findButton);
            }
            if (buttonKeys[i].equals("replfind")) {
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        find();
                    }
                });
                panel.add(button);
            }
            else if (buttonKeys[i].equals("replace")) {
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        replace(0);
                    }
                });
                replaceButton = button;
                panel.add(replaceButton);
            }
            else if (buttonKeys[i].equals("replaceAll")) {
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        replace(1);
                    }
                });
                panel.add(button);
            }
            else if (buttonKeys[i].equals("goto")) {
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        gotoLine();
                    }
                });
                gotoButton = button;
                panel.add(gotoButton);
            }
            else if (buttonKeys[i].equals("close")) {
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        close();
                    }
                });
                panel.add(button);
            }
        }
        return panel;
    }

    private void find() {
        String search = "";
        if (tabp.getSelectedIndex() == 0) {
            search = findInput.getText();
        }
        else if (tabp.getSelectedIndex() == 1) {
            search = replaceFindInput.getText();
        }
        if (searchFor.equals("") || !searchFor.equals(search)) {
            if (!case_sensitive.isSelected()) {
                if (!searchFor.equals(search.toLowerCase())) {
                    start_offset = 0;
                }
            }
            else {
                start_offset = 0;
            }
        }
        try {
            String content = source.getDocument().getText(0, source.getDocument().getLength());
            if (HyperPanel.jTabPane.getSelectedIndex() == 0) {
                content = content.substring(HyperPanel.guidePositions[0]);
            }
            if (!case_sensitive.isSelected()) {
                content = content.toLowerCase();
                searchFor = search.toLowerCase();
            }
            else {
                searchFor = search;
            }
            if (!searchFor.equals("") && !searchFor.equals(" ")) {
                if (search_backward.isSelected()) {
                    if (content.lastIndexOf(searchFor, start_offset-searchFor.length()) == -1 && !secondTimeNotFound) {
                        if (makeDocumentDialog("begin")) {
                            start_offset = source.getDocument().getLength();
                            secondTimeNotFound = true;
                            find();
                        }
                    }
                    else if (content.lastIndexOf(searchFor, start_offset-searchFor.length()) == -1 && secondTimeNotFound) {
                        makeErrorDialog();
                        secondTimeNotFound = false;
                    }
                    else if (content.lastIndexOf(searchFor, start_offset) > -1) {
                        start_offset -= searchFor.length();
                        start_offset = content.lastIndexOf(searchFor, start_offset);
                        source.setSelectionStart(start_offset);
                        source.setSelectionEnd(start_offset+searchFor.length());
                        secondTimeNotFound = false;
                    }
                }
                else {
                    if (content.indexOf(searchFor, start_offset) > -1) {
                        start_offset = content.indexOf(searchFor, start_offset);
                        if (HyperPanel.jTabPane.getSelectedIndex() == 0) {
                            start_offset = start_offset + HyperPanel.guidePositions[0];
                        }
                        source.setSelectionStart(start_offset);
                        source.setSelectionEnd(start_offset+searchFor.length());
                        start_offset += searchFor.length();
                    }
                    else if (content.indexOf(searchFor, start_offset) == -1 && start_offset == 0) {
                        makeErrorDialog();
                    }
                    else {
                        if (makeDocumentDialog("end")) {
                            start_offset = 0;
                            find();
                        }
                    }
                }
            }
        } catch (Exception ex) {}
    }

    private void replace(int type) {
        int occurence = 0;
        int startOffset = 0;
        if (HyperPanel.jTabPane.getSelectedIndex() == 0) {
            startOffset = HyperPanel.guidePositions[0];
        }
        if (replaceFindInput.getText().length() != 0) {
            find();
            do {
                if (source.getSelectedText() != null) {
                    source.replaceSelection(replaceInput.getText());
                    occurence += 1;
                    if (type == 1) {
                        find();
                    }
                }
            }
            while (type == 1 && source.getSelectedText() != null);
            if (type == 1 && occurence > 0) {
                makeReplacedDialog(occurence);
            }
        }
    }

    private void gotoLine() {
        try {
            int gotoLine = Integer.valueOf(gotoInput.getText()).intValue();
            source.setCaretPosition(linePosition(gotoLine));
            close();
        } catch (Exception ex) {
            HyperOptionPane.showDialog(frame, 
                                       HyperPanel.getResourceString("gotoNotNumericErrorMessageWindowTitle"),
                                       HyperPanel.getResourceString("gotoNotNumericErrorMessage"),
                                       HyperOptionPane.DEFAULT_OPTION);
        }
    }
    
    private void inicialize(int index) {
        switch (index) {
        case 0: findInput.requestFocus();
            getRootPane().setDefaultButton(findButton);
            break;
        case 1: replaceFindInput.requestFocus();
            getRootPane().setDefaultButton(replaceButton);
            break;
        case 2: gotoInput.requestFocus();
            getRootPane().setDefaultButton(gotoButton);
        }
    }
    
    private boolean makeDocumentDialog(String key) {
        int choice = 0;
        if (key.equals("end")) {
            choice = HyperOptionPane.showDialog(frame, 
                                                "HyperPad: "+HyperPanel.getResourceString("documentMessageWindowTitle"),
                                                HyperPanel.getResourceString("documentEndMessage"),
                                                HyperOptionPane.YES_NO_OPTION);
        }
        else if (key.equals("begin")) {
            choice = HyperOptionPane.showDialog(frame, 
                                                "HyperPad: "+HyperPanel.getResourceString("documentMessageWindowTitle"),
                                                HyperPanel.getResourceString("documentBeginMessage"),
                                                HyperOptionPane.YES_NO_OPTION);
        }
        if (choice == HyperOptionPane.YES) {
            return true;
        }
        else {
            return false;
        }
    }

    private void makeErrorDialog() {
        HyperOptionPane.showDialog(frame, 
                                   "HyperPad: "+HyperPanel.getResourceString("notFoundErrorMessageWindowTitle"),
                                   HyperPanel.getResourceString("notFoundErrorMessage"), 
                                   HyperOptionPane.DEFAULT_OPTION);
    } 

    private void makeReplacedDialog(int foundNumber) {
        HyperOptionPane.showDialog(frame, 
                                   "HyperPad: "+HyperPanel.getResourceString("replaceMessageWindowTitle"),
                                   foundNumber + " " + HyperPanel.getResourceString("replaceMessage"),
                                   HyperOptionPane.DEFAULT_OPTION);
    }

    private int countLineNumber() {
        int lineNumber = 1;
        String content = "";
        try {
            content = source.getDocument().getText(0, source.getDocument().getLength());
        } catch (Exception ex) {}
        int offset = 0;
        while (content.indexOf("\n", offset+1) > -1) {
            offset = content.indexOf("\n", offset+1);
            lineNumber += 1;
        }
        return lineNumber;
    }

    private int linePosition(int gotoLine) {
        int lineNumber = 1;
        String content = "";
        try {
            content = source.getDocument().getText(0, source.getDocument().getLength());
        } catch (Exception ex) {}
        int offset = 0;
        while (content.indexOf("\n", offset+1) > -1 && gotoLine > lineNumber) {
            offset = content.indexOf("\n", offset+1);
            lineNumber += 1;
        }
        return offset+1;
    }

    private void setResult(boolean value) {
        result = value;
    }

    private void close() { 
        this.dispose();
    }

    final String findButtons = "find close";
    final String replaceButtons = "replfind replace replaceAll close";
    final String gotoButtons = "goto close";
}
