/*
 * $COPYRIGHT$
 * $Id: HelpBox.java,v 1.10 2002/01/23 17:22:35 borzy Exp $
 *
 * Date        Author               Changes
 * Jul 10 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.net.URL;
import java.io.IOException;
import java.util.Vector;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.ActionListener;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.BorderFactory;
import javax.swing.SwingConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent.EventType;
import org.media.hyperpad.gui.HypTextPane;
import org.media.hyperpad.conf.ConfigHandler;
/**
 * Class meant to construct the HelpBox for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.10 $ $Date: 2002/01/23 17:22:35 $
 * @see String
 */  
public class HelpBox extends JDialog {
    ConfigHandler ch;
    Vector historyList = new Vector(20);
    final HypTextPane tp = new HypTextPane(HypTextPane.NO_LINE_WRAP);
    JFrame frame;
    boolean linkStatus = true;
    int historyPosition = 0;
    JButton back = new JButton(MyResources.getImageIcon("backHelpImage"));
    JButton forw = new JButton(MyResources.getImageIcon("forwHelpImage"));

    public HelpBox(final JFrame f) {
        super(f, "", true);
        this.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent ke) {
                if(ke.getKeyCode() == KeyEvent.VK_ESCAPE){
                    close();
                }
            }
            public void keyReleased(KeyEvent ke) {}
            public void keyTyped(KeyEvent ke) {}
        });
        frame = f;
        ch = new ConfigHandler(HyperPanel.XMLpath);
        JPanel container = new JPanel(new BorderLayout());
        JPanel button_panel = new JPanel(new FlowLayout(FlowLayout.LEFT,6,0));
        JButton home = new JButton(MyResources.getImageIcon("homeImage"));
        button_panel.add(back);
        button_panel.add(home);
        button_panel.add(forw);
        back.setEnabled(false);
        forw.setEnabled(false);
        home.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    tp.setPage(ClassLoader.getSystemResource(MyResources.getTextValue("helpHTML")));
                }
                catch (Exception ex) {}
            }
        });
        back.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    historyPosition -= 1;
                    tp.setPage((URL)historyList.elementAt(historyPosition));
                    updateButtons();
                }
                catch (Exception ex) {}
            }
        });
        forw.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    historyPosition += 1;
                    updateButtons();
                    tp.setPage((URL)historyList.elementAt(historyPosition)); 
                }
                catch (Exception ex) {}
            }
        });
        container.add(button_panel, BorderLayout.NORTH);
        JScrollPane sp = new JScrollPane();
        try {
            historyList.add(ClassLoader.getSystemResource(MyResources.getTextValue("helpHTML")));
            tp.setPage((URL)historyList.firstElement());
        }
        catch (Exception ex) {}
        tp.addHyperlinkListener(new HyperlinkListener () {
            public void hyperlinkUpdate(final HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ENTERED) {
                    linkStatus = false;
                }
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED && !linkStatus) {
                    updateList();
                    URL url = e.getURL();
                    if (!url.toString().startsWith("http://")) {
                        historyList.add(url);
                        historyPosition += 1;
                        updateButtons();
                        linkStatus = true;
                        if (linkStatus) {
                            setPageToURL(url);
                        }
                    }
                    else {
                        setPageToURL(url);
                    }
                }
            }
        });
        tp.setEditable(false);
        tp.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        sp.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        sp.getViewport().add(tp);
        container.add(sp, BorderLayout.CENTER);
        JPanel buttonPanel = new JPanel(new BorderLayout());
        JButton button = new JButton(MyResources.getTextValue("aboutBoxButton"));
        button.setFont(ch.getMenuFont());
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                close();
            }
        });
        buttonPanel.add(button, BorderLayout.EAST);
        container.add(buttonPanel, BorderLayout.SOUTH);
        container.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        getContentPane().add(container);
        setResizable(true);
        setTitle("Help");
        setModal(false);
        setSize(500,600);
        setLocationRelativeTo(f);
    }

    private void updateList() {
        if (historyPosition < historyList.size()-1) {
            while(historyPosition+1 < historyList.size()) {
                historyList.removeElementAt(historyPosition+1);
            }
        }
    }

    private void updateButtons() {
        if (historyPosition == 0) {
            back.setEnabled(false);
            forw.setEnabled(true);
        }
        else if (historyPosition == historyList.size()-1) {
            back.setEnabled(true);
            forw.setEnabled(false);
        }
        else {
            back.setEnabled(true);
            forw.setEnabled(true);
        }
    }

    private void setPageToURL(URL url) {
        if (url.toString().startsWith("http://")) {
            try {
                java.lang.Runtime.getRuntime().exec(HyperPanel.ch.getBrowser() + " " + url);
            } catch (Exception ex) {
                HyperOptionPane.showDialog (frame,
                                            HyperPanel.getResourceString("invalidBrowserErrorMessageWindowTitle"),
                                            HyperPanel.getResourceString("invalidBrowserErrorMessage"),
                                            HyperOptionPane.DEFAULT_OPTION); 
            }
        }
        else {
            try {
                tp.setPage(url);
            } catch (IOException io) {
                historyList.removeElementAt(historyPosition);
                historyPosition -= 1;
                if (historyPosition == 0) {
                    back.setEnabled(false);
                    forw.setEnabled(false);
                }
                else {
                    updateButtons();
                }
                HyperOptionPane.showDialog (frame,
                                            HyperPanel.getResourceString("invalidLinkErrorMessageWindowTitle"),
                                            HyperPanel.getResourceString("invalidLinkErrorMessage"),
                                            HyperOptionPane.DEFAULT_OPTION); 
            }
        }
    }

    private void close() {
        this.dispose();
    }
}
