/*
 * $COPYRIGHT$
 * $Id: HyperDocument.java,v 1.43 2002/04/23 19:31:52 borzy Exp $
 *
 * Date        Author               Changes
 * May 28 2001 Borsos Szabolcs      Created
 */     
package org.media.hyperpad;

import javax.swing.text.Element;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.html.StyleSheet;
import javax.swing.text.html.HTMLDocument;
import java.util.Vector;
import java.util.StringTokenizer;
import java.awt.Font;
import java.awt.Color;
/**
 * Class needed by HyperPad for syntax highlighting.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.43 $ $Date: 2002/04/23 19:31:52 $
 * @see String
 */ 
public class HyperDocument extends HTMLDocument {
    public static HyperEditorKit hek=null;
    static int bracketsNR = 0;
    private boolean _isString = false;
    private boolean _isComment = false;

    public HyperDocument (StyleSheet ss) {
        super(ss);
        addUndoableEditListener(new UndoHandler());
    }

    protected static Vector getKeywords() {
        Vector keywords = new Vector();
        keywords.add("this");
        keywords.add("simplex");
        keywords.add("regexp");
        keywords.add("static");
        keywords.add("nill");
        keywords.add("options");
        keywords.add("filter");
        keywords.add("limit");
        keywords.add("requests");
        keywords.add("depth");
        keywords.add("timeout");
        keywords.add("if");
        keywords.add("then");
        keywords.add("elif");
        keywords.add("else");
        keywords.add("switch");
        keywords.add("where");
        keywords.add("break");
        keywords.add("by");
        keywords.add("first-of");
        keywords.add("last-of");
        keywords.add("find");
        keywords.add("first");
        keywords.add("last");
        keywords.add("include");
        keywords.add("true");
        keywords.add("false");
        keywords.add("super");
        keywords.add("and");
        keywords.add("or");
        keywords.add("exit");
        keywords.add("in");
        keywords.add("each");
        keywords.add("print");
        keywords.add("define");
        keywords.add("extends");
        keywords.add("label");
        keywords.add("typeof");
        keywords.add("to");
        keywords.add("from");
        keywords.add("while");
        keywords.add("do");
        keywords.add("return");
        return keywords;
    }

    protected static Vector getSeparators() {
        Vector separators = new Vector();
        separators.add("\\");
        separators.add("\"");
        separators.add("#");
        separators.add(" ");
        separators.add("\n");
        separators.add("=");
        separators.add("[");
        separators.add("]");
        return separators;
    }

    protected String getDelimiters() {
        String delimiters = "";
        for (int i=0; i<getSeparators().size(); i++) {
            delimiters = delimiters.concat((String)getSeparators().elementAt(i));
        }
        return delimiters;
    }

    public void insertString(int offset, String inserted_text, AttributeSet attr) {
        try {
            if (inserted_text.startsWith(ResultPrintStream.RESULT)) {
                resultTokenizer(offset, inserted_text.substring(ResultPrintStream.RESULT.length()));
            }
            else if (HyperPanel.guidePositions[0] <= offset) {
                updateSaveStatus();
                super.insertString(offset, inserted_text, getAttributeSet(attr));
                verifyBracketsNR(inserted_text, INSERT);
                if (HyperPanel.ch.getHighlight()) {
                    if (inserted_text.lastIndexOf("\n") != -1) {
                        HyperPanel.documentLength = getLength();
                        UndoHandler.offset = offset;
                        insertTokenizer(offset, inserted_text);
                    }
                    else {
                        int start = readBackToFirstSpecialChar(offset, "\n");
                        int end = getText(0, getLength()).indexOf("\n", offset);
                        if (end == -1 || start == end) {
                            end = getLength();
                        }
                        HyperPanel.documentLength = getLength();
                        UndoHandler.offset = offset;
                        insertTokenizer(start, getText(0, getLength()).substring(start, end));
                    }
                }
            }
            else {
                insertString(HyperPanel.guidePositions[1], inserted_text, getAttributeSet(attr));
            }
        } catch (Exception ex) {}
    }

    public void remove(int offset, int length) {
        try {
            if (HyperPanel.guidePositions[0] <= offset) {
                updateSaveStatus();
                verifyBracketsNR(getText(0, getLength()).substring(offset, offset+length), REMOVE);
                super.remove(offset, length);
                int start = readBackToFirstSpecialChar(offset-1, "\n");
                int end = getText(0, getLength()).indexOf("\n", offset);
                if (end == -1 || start == end) {
                    end = getLength();
                }
                if (HyperPanel.ch.getHighlight()) {
                    if (offset > HyperPanel.prompt.length()) {
                        HyperPanel.documentLength = getLength();
                        UndoHandler.offset = offset;
                        insertTokenizer(start, getText(0, getLength()).substring(start, end));
                    }
                }
            }
            else {
                HyperPanel.pane.setCaretPosition(HyperPanel.guidePositions[1]);
            }
        } catch (Exception ex) {}
    }

    private void insertTokenizer(int offset, String text) {
        boolean isString = false;
        boolean isNumeric = false;
        boolean isComment = false;
        boolean isEscChar = false;
        int stringStart = 0;
        StringTokenizer t = new StringTokenizer(text, getDelimiters(), true);
        int token_number = t.countTokens();
        for (int i=0; i < token_number; i++) {
            String token = t.nextToken();
            if (token.equals("\n")) {
                isString = false;
                isComment = false;
            }
            else if (token.equals("\\")) {
                isEscChar = true;
            }
            if (token.equals("\"") && !isComment) {
                if (!isEscChar) {
                    if (isString) {
                        isString = false;
                    } else {
                        isString = true;
                        stringStart = offset;
                    }
                    offset += token.length();
                }
                else {
                    isEscChar = false;
                }
            }
            else if ((token.equals("#") || isComment) && !isString) {
                isComment = true;
                if (!isString) {
                    makeComment(offset, token.length(), null);
                }
                offset += token.length();
            }
            else {
                if (!isString) {
                    evaluateWord(offset, token);
                }
                offset += token.length();
            }
            if (isString && !isComment) {
                makeString(stringStart, offset-stringStart+1, null);
            }
            _isString = isString;
            _isComment = isComment;
        }
    }

    private void resultTokenizer(int offset, String text) {
        boolean isLink = false;
        boolean isTable = false;
        boolean spaceLock = false;
        boolean htmlResultStart = false;
        int spaceNR = 0;
        int start = getEndPosition().getOffset()-1;
        String inserted = "";
        String verify = "";
        String buffer, tag, txt;

        if (HyperPanel.ch.getHTMLMode() && text.indexOf("<html>") != -1) {
            text = verifyHTML(text);
            htmlResultStart = true;
        }
        try {
            insertBeforeEnd(getParagraphElement(offset-1), text);
            inserted = getText(start, getEndPosition().getOffset()-start);
        } catch (Exception ex) {}
        StringTokenizer t = new StringTokenizer(text, "<>", true);
        while (t.hasMoreTokens()) {
            String token = t.nextToken();
            if (token.equals("<")) {
                buffer = "<";
                verify = "";
                while (!verify.equals(">")) {
                    verify = t.nextToken();
                    buffer += verify;
                }
                txt = buffer.substring(0, buffer.lastIndexOf("<"));
                tag = buffer.substring(buffer.lastIndexOf("<")+1);
            }
            else {
                txt = token;
                tag = "";
            }
            if (!tag.equals("")) {
                if (tag.trim().toLowerCase().startsWith("body")) {
                    htmlResultStart = false;
                    offset = 0;
                }
                else if (tag.trim().toLowerCase().startsWith("a")) {
                    if (tag.substring(tag.toLowerCase().indexOf("a")+1).trim().toLowerCase().indexOf("href") != -1) {
                        isLink = true;
                    }
                    else {
                        isLink = false;
                    }
                }
                else if (tag.trim().toLowerCase().startsWith("table")) {
                    isTable = true;
                }
                else if (tag.trim().startsWith("/")) {
                    if (tag.substring(tag.indexOf("/")+1).trim().toLowerCase().startsWith("a")) {
                        isLink = false;
                    }
                    else if (tag.substring(tag.indexOf("/")+1).trim().toLowerCase().startsWith("table")) {
                        isTable = false;
                    }
                }
            }
            else if (tag.equals("") && isLink && !htmlResultStart) {
                if (txt.trim().length() != 0) {
                    if (txt.indexOf("&") != -1 && txt.indexOf(";") != -1) {
                        txt = displayedText(txt);
                    }
                    if (txt.indexOf(" ") != -1 || txt.indexOf((char)160) != -1) {
                        boolean add = true;
                        buffer = "";
                        StringTokenizer linkTokenizer = new StringTokenizer(txt, " ", true);
                        int tokenNR = linkTokenizer.countTokens();
                        for (int j=0; j < tokenNR; j++) {
                            String word = linkTokenizer.nextToken();
                            if (word.equals(" ") && add) {
                                add = false;
                                buffer = buffer + word;
                            }
                            else if (!word.equals(" ")) {
                                add = true;
                                buffer = buffer + word;
                            }
                        }
                        txt = buffer;
                    }
                    makeLink(inserted.indexOf(txt, offset-start-1)+start, txt.length(), null);
                    offset += txt.length();
                }
            }
            else if (tag.equals("") && !htmlResultStart) {
                if (txt.indexOf("&") != -1 && txt.indexOf(";") != -1) {
                    txt = displayedText(txt);
                }
                StringTokenizer st = new StringTokenizer(txt, " ", true);
                int tokenNR = st.countTokens();
                for (int j=0; j < tokenNR; j++) {
                    String word = st.nextToken();
                    if (isLink(word)) {
                        if (word.endsWith(".") || word.endsWith(",") || word.endsWith(":") || word.endsWith(";")) {
                            makeLink(inserted.indexOf(word, offset-start-1)+start, word.length()-1, null);
                        }
                        else {
                            makeLink(inserted.indexOf(word, offset-start-2)+start, word.length(), null);
                        }
                        if (isTable) {
                            offset += word.length()+1;
                        }
                        else {
                            offset += word.length();
                        }
                        spaceNR = 0;
                    }
                    else {
                        if (word.indexOf("\r\n") != -1) {
                            if (word.equals("\r\n")) {
                                spaceLock = true;
                            }
                            else if (word.startsWith("\r\n")) {
                                spaceLock = false;
                            }
                            else if (word.endsWith("\r\n")) {
                                spaceLock = true;
                                spaceNR--;
                            }
                            else {
                                spaceNR = 0;
                            }
                            offset += word.length()-1-spaceNR;
                            spaceNR = 0;
                        }
                        else if (word.indexOf("\n") != -1 || word.indexOf("\r") != -1 || word.indexOf("\t") != -1) {
                            if (word.equals("\n") || word.equals("\r") || word.equals("\t")) {
                                if (spaceLock) spaceNR = 1;
                                else spaceLock = true;
                            }
                            else if (word.startsWith("\n") || word.startsWith("\r") || word.startsWith("\t")) {
                                if (spaceLock) spaceNR = 1;
                                spaceLock = false;
                            }
                            else if (word.endsWith("\n")  || word.endsWith("\r") || word.endsWith("\t")) {
                                if (spaceLock) spaceNR = 2;
                                spaceLock = true;
                                spaceNR--;
                            }
                            else {
                                spaceNR = 0;
                            }
                            offset += word.length()-spaceNR;
                            spaceNR = 0;
                        }
                        else if (word.equals(" ")) {
                            if (!spaceLock) {
                                spaceNR++;
                                offset += word.length();
                            }
                        }
                        else {
                            spaceNR = 0;
                            spaceLock = false;
                            offset += word.length();
                        }
                    }
                }
            }
        }
    }

    private SimpleAttributeSet getAttributeSet(AttributeSet attr) {
        SimpleAttributeSet sas;
        if (attr != null) {
            sas = new SimpleAttributeSet(attr);
        }
        else {
            sas = new SimpleAttributeSet();
        }
        return sas;
    }

    private void setAttributes(int offset, int length, SimpleAttributeSet sas) {
        if (offset > HyperPanel.prompt.length()) {
            setCharacterAttributes(offset, length, sas, false);
        }
        else {
            setCharacterAttributes(HyperPanel.prompt.length(), length, sas, false);
        }
    }

    protected void insertPrompt() {
        try {
            Element elem = getParagraphElement(getEndPosition().getOffset());
            insertBeforeEnd(elem, HyperPanel.prompt);
            HyperPanel.guidePositions[0] = getLength();
            HyperPanel.uh.undo.discardAllEdits();
            HyperPanel.uh.updateUndo();
            HyperPanel.uh.updateRedo();
        } catch (Exception ex) {}
    }

    private void updateSaveStatus() {
        if (HyperPanel.iactivDocSaved) {
            HyperPanel.iactivDocSaved = false;
            HyperPanel.updateSaveButton();
        }
    }

    private void makeKeyword(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.keywordColor));
        setAttributes(offset, length, sas);
    }

    private void makeString(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.stringColor));
        setAttributes(offset, length, sas);
    }

    private void makeNormal(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.generalColor));
        setAttributes(offset, length, sas);
    }

    private void makeComment(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.commentColor));
        setAttributes(offset, length, sas);
    }

    private void makeNumeric(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.numericColor));
        setAttributes(offset, length, sas);
    }

    private void makeLink(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setForeground(sas, Color.blue);
        StyleConstants.setUnderline(sas, true);
        setAttributes(offset, length, sas);
    }

    private int readBackToFirstSpecialChar(int offset, String specialChar) {
        int firstSpecialChar = 0;
        int commandStart = 0;
        String prevChar = "";
        try {
            prevChar = getText(0, getLength()).substring(offset, offset+1);
            String before = getText(0, getLength()).substring(0, offset+1);
            commandStart = before.lastIndexOf(HyperPanel.prompt)+HyperPanel.prompt.length();
        } catch (BadLocationException blex) {
            blex.printStackTrace();
        }
        if (specialChar == null) {
            for (int i=0; i<getSeparators().size(); i++) {
                if (prevChar.equals((String)getSeparators().elementAt(i))) {
                    firstSpecialChar = offset;
                }
            }
        }
        else if (specialChar.equals(prevChar)) {
            firstSpecialChar = offset;
        }    
        if (firstSpecialChar != 0) {
            return firstSpecialChar;
        }
        else if (offset > commandStart) {
            return readBackToFirstSpecialChar(offset-1, specialChar);
        }
        else {
            return commandStart;
        }
    }

    private int readForwToFirstSpecialChar(int offset, String specialChar) {
        int firstSpecialChar = 0;
        String nextChar = "";
        if (offset+1 < getLength()) {
            try {
                nextChar = getText(0, getLength()).substring(offset+1, offset+2);
            } catch (Exception ex) {}
        }
        if (specialChar == null) {
            for (int i=0; i<getSeparators().size(); i++) {
                if (nextChar.equals((String)getSeparators().elementAt(i))) {
                    firstSpecialChar = offset;
                }
            }
        }
        else {
            if (nextChar.equals(specialChar)) {
                firstSpecialChar = offset;
            }
        }
        if (firstSpecialChar != 0) {
            if (firstSpecialChar < getLength()) {
                return firstSpecialChar+1;
            }
            else {
                return firstSpecialChar;
            }
        }
        else if (offset+1 < getLength()) {
            return readForwToFirstSpecialChar(offset+1, specialChar);
        }
        else {
            return getLength();
        }
    }

    private String lastCommand() {
        String lastCommand = "";
        String all = "";
        try {
            all = getText(0, getLength());
        } catch (Exception ex) {}
        if (all.length() > HyperPanel.prompt.length()) {
            int commandStart = all.lastIndexOf(HyperPanel.prompt);
            lastCommand = all.substring(commandStart, getLength());
        }
        return lastCommand;
    }

    protected boolean isNumeric(String arg) {
        try {
            float test = Float.valueOf(arg).floatValue();
            return true;
        } catch (NumberFormatException nfex) {
            return false;
        }
    }

    private boolean isLink(String arg) {
        if (arg.endsWith(".") || arg.endsWith(",") || arg.endsWith(":") || arg.endsWith(";")) {
            arg = arg.substring(0, arg.length()-1);
        }
        try {
            java.net.URL url = new java.net.URL(arg);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    private String displayedText(String text) {
        try {
            javax.swing.text.html.HTMLEditorKit kit = new javax.swing.text.html.HTMLEditorKit();
            HTMLDocument htmlDoc = (HTMLDocument)kit.createDefaultDocument();
            kit.insertHTML(htmlDoc, 0, text, 0, 0, javax.swing.text.html.HTML.Tag.HTML);
            return htmlDoc.getText(0, htmlDoc.getLength()-1);
        } catch (Exception ex) {
            return "";
        }
    }

    private String verifyHTML(String html) {
        String result = "";
        String buffer, test, Tag, notTag;
        boolean canAdd = true;
        boolean isBody = false;
        StringTokenizer st = new StringTokenizer(html, "<>", true);
        while (st.hasMoreTokens()) {
            String token = st.nextToken();
            if (token.equals("<")) {
                buffer = "<";
                test = "";
                while (!test.equals(">")) {
                    test = st.nextToken();
                    buffer += test;
                }
                notTag = buffer.substring(0, buffer.lastIndexOf("<"));
                Tag = buffer.substring(buffer.lastIndexOf("<")+1);
            }
            else {
                notTag = token;
                Tag = "";
            }
            if (!Tag.equals("")) {
                if (Tag.trim().toLowerCase().startsWith("body")) {
                    isBody = true;
                }
                if (Tag.trim().toLowerCase().startsWith("script") ||
                    Tag.trim().toLowerCase().startsWith("applet") ||
                    Tag.trim().toLowerCase().startsWith("noscript")) {
                    canAdd = false;
                }
                else if (Tag.trim().startsWith("/")) {
                    if (Tag.substring(Tag.indexOf("/")+1).trim().toLowerCase().startsWith("script") ||
                        Tag.substring(Tag.indexOf("/")+1).trim().toLowerCase().startsWith("applet") ||
                        Tag.substring(Tag.indexOf("/")+1).trim().toLowerCase().startsWith("noscript")) {
                        canAdd = true;
                        Tag = "";
                    }
                }
            }
            if (canAdd && isBody) {
                if (!Tag.equals("")) {
                    Tag = "<" + Tag;
                }
                result += Tag + notTag;
            }
        }
        return "<html>" + result;
    }

    private void evaluateWord(int offset, String word) {
        if (getKeywords().contains(word.toLowerCase())) {
            makeKeyword(offset, word.length(), null);
        }
        else if (isNumeric(word)) {
            makeNumeric(offset, word.length(), null);
        }
        else {
            makeNormal(offset, word.length(), null);
        }
    }

    private void verifyBracketsNR(String text, int type) {
        if (text.length() == 1) {
            if (text.equals("[") && !_isString && !_isComment) {
                if (type == REMOVE) {
                    bracketsNR--;
                }
                else {
                    bracketsNR++;
                }
            }
            else if (text.equals("]") && !_isString && !_isComment) {
                if (type == REMOVE) {
                    bracketsNR++;
                }
                else {
                    bracketsNR--;
                }
            }
        }
        else {
            StringTokenizer st = new StringTokenizer(text, "[]", true);
            int tokenNR = st.countTokens();
            for (int i=0; i < tokenNR; i++) {
                String token = st.nextToken();
                if (token.equals("[") && !_isString && !_isComment) {
                    if (type == REMOVE) {
                        bracketsNR--;
                    }
                    else {
                        bracketsNR++;
                    }
                }
                else if (token.equals("]") && !_isString && !_isComment) {
                    if (type == REMOVE) {
                        bracketsNR++;
                    }
                    else {
                        bracketsNR--;
                    }
                }
            }
        }
    }

    private final int INSERT = 1;
    private final int REMOVE = 0;
}
