/*
 * $COPYRIGHT$
 * $Id: HyperOptionPane.java,v 1.5 2001/12/20 21:57:28 borzy Exp $
 *
 * Date        Author               Changes
 * Jul 02 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.util.Vector;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JComponent;
import javax.swing.BorderFactory;
import org.media.hyperpad.gui.HypLabel;
import org.media.hyperpad.gui.HypButton;
import org.media.hyperpad.gui.HypTextField;
import org.media.hyperpad.gui.HypComboBox;
/**
 * Class meant to show different OptionPane for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.5 $ $Date: 2001/12/20 21:57:28 $
 * @see String
 */  
public class HyperOptionPane extends JDialog {
    static HypComboBox combo = null;
    static String selectedItem = null;
    static int DEFAULT_OPTION = 0;
    static int YES_NO_OPTION = 1;
    static int YES_NO_CANCEL_OPTION = 2;
    static int OK_CANCEL_OPTION = 3;
    static int EMPTY_OPTION = 4;
    static int result = 0;
    static int NO = 0;
    static int YES = 1;
    static int CANCEL = 2;
    
    public HyperOptionPane(JFrame frame, String title, String label, 
                           JComponent comp, int messageType) {
        super(frame, "", true);
        this.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent ke) {
                if(ke.getKeyCode() == KeyEvent.VK_ESCAPE){
                    close();
                }
            }
        });
        HypLabel message = new HypLabel(label);
        message.setFont(HyperPanel.ch.getMenuFont());
        message.setForeground(Color.black);
        message.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        JPanel main = new JPanel();
        if (comp != null) {
            main.add(comp);
        }
        main.add(message);
        main.setAlignmentX(javax.swing.SwingConstants.CENTER);
        main.setBorder(BorderFactory.createEmptyBorder(0,6,11,6));
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(message, BorderLayout.NORTH);
        getContentPane().add(main, BorderLayout.CENTER);
        getContentPane().add(makeButtonPanel(messageType), BorderLayout.SOUTH);
        setTitle(title);
        setModal(true);
        pack();
        setLocationRelativeTo(frame);
        setVisible(true);
    }

    public static int showDialog(JFrame frame, String title, String label, int messageType) {
        HyperOptionPane hop = new HyperOptionPane(frame, title, label, null, messageType);
        return result;
    }

    public static String showComboDialog(JFrame frame, String title, String label, Vector elements) {
        combo = new HypComboBox(elements);
        HyperOptionPane hop = new HyperOptionPane(frame, title, label, combo, OK_CANCEL_OPTION);
        return selectedItem;
    }

    public static String showInputDialog(JFrame frame, String title, String label, String textfieldLabel) {
        JPanel panel = new JPanel(new BorderLayout());
        if (null != label || !label.equals("")) {
            HypLabel tfLabel = new HypLabel(textfieldLabel);
            tfLabel.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
            panel.add(tfLabel, BorderLayout.NORTH);
        }
        HypTextField input = new HypTextField(20);
        panel.add(input, BorderLayout.CENTER);
        HyperOptionPane hop = new HyperOptionPane(frame, title, label, panel, OK_CANCEL_OPTION);
        return input.getText();
    }

    private JPanel makeButtonPanel(int type) {
        JPanel bpanel = new JPanel();
        if (type == DEFAULT_OPTION) {
            HypButton ok = new HypButton(HyperPanel.getResourceString("okButtonLabel"));
            ok.setFont(HyperPanel.ch.getMenuFont());
            ok.setForeground(Color.black);
            ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    close();
                }
            });   
            bpanel.setLayout(new FlowLayout(FlowLayout.CENTER));
            bpanel.add(ok);
            bpanel.setBorder(BorderFactory.createEmptyBorder(0,6,6,6));
            this.getRootPane().setDefaultButton(ok);
        }
        else if (type == YES_NO_OPTION || type == OK_CANCEL_OPTION) {
            HypButton yes, no;
            if (type == YES_NO_OPTION) {
                yes = new HypButton(HyperPanel.getResourceString("yesButtonLabel"));
                no = new HypButton(HyperPanel.getResourceString("noButtonLabel"));
            }
            else {
                yes = new HypButton(HyperPanel.getResourceString("okButtonLabel"));
                no = new HypButton(HyperPanel.getResourceString("cancelButtonLabel"));
            }
            yes.setFont(HyperPanel.ch.getMenuFont());
            yes.setForeground(Color.black);
            yes.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (combo != null) {
                        selectedItem = (String)combo.getSelectedItem();
                    }
                    setResult(YES);
                    close();
                }
            });   
            no.setFont(HyperPanel.ch.getMenuFont());
            no.setForeground(Color.black);
            no.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setResult(NO);
                    selectedItem = null;
                    close();
                }
            }); 
            bpanel.setLayout(new FlowLayout(FlowLayout.CENTER));
            bpanel.add(yes);
            bpanel.add(no);
            this.getRootPane().setDefaultButton(yes);
        }
        else if (type == YES_NO_CANCEL_OPTION) {
            HypButton yes = new HypButton(HyperPanel.getResourceString("yesButtonLabel"));
            HypButton no = new HypButton(HyperPanel.getResourceString("noButtonLabel"));
            HypButton cancel = new HypButton(HyperPanel.getResourceString("okButtonLabel"));
            yes.setFont(HyperPanel.ch.getMenuFont());
            yes.setForeground(Color.black);
            yes.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setResult(YES);
                    close();
                }
            });   
            no.setFont(HyperPanel.ch.getMenuFont());
            no.setForeground(Color.black);
            no.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setResult(NO);
                    close();
                }
            }); 
            cancel.setFont(HyperPanel.ch.getMenuFont());
            cancel.setForeground(Color.black);
            cancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setResult(CANCEL);
                    close();
                }
            }); 
            bpanel.setLayout(new FlowLayout(FlowLayout.CENTER));
            bpanel.add(yes);
            bpanel.add(no);
            bpanel.add(cancel);
            this.getRootPane().setDefaultButton(yes);
        }
        else if (type == EMPTY_OPTION) {
        }
        return bpanel;
    }
            
    private void setResult(int value) {
        result = value;
    }

    public void close() {
        this.dispose();
    }
}
