/*
 * $COPYRIGHT$
 * $Id: HyperSourceDocument.java,v 1.9 2001/11/04 18:03:44 borzy Exp $
 *
 * Date        Author               Changes
 * Jul 24 2001 Borsos Szabolcs      Created
 */     
package org.media.hyperpad;

import javax.swing.text.Element;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.html.StyleSheet;
import javax.swing.text.html.HTMLDocument;
import java.util.Vector;
import java.util.StringTokenizer;
import java.awt.Font;
import java.awt.Color;
/**
 * Class needed by HyperPad Source Editor for syntax highlighting.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.9 $ $Date: 2001/11/04 18:03:44 $
 * @see String
 */ 
public class HyperSourceDocument extends HTMLDocument {
    static boolean isLongInsert = false;
    static HyperEditorKit shek=null;
    HyperDocument hdoc = new HyperDocument(shek.getDefaultStyle());

    public HyperSourceDocument (StyleSheet ss) {
        super(ss);
        addUndoableEditListener(new UndoHandler());
    }

    public void insertString(int offset, String inserted_text, AttributeSet attr) {
        try {
            super.insertString(offset, inserted_text, attr);
            updateSaveStatus();
            if (HyperPanel.ch.getHighlight()) {
                if (inserted_text.lastIndexOf("\n") != -1) {
                    HyperPanel.sourceDocumentLength = getLength();
                    UndoHandler.sourceOffset = offset;
                    insertTokenizer(offset, inserted_text);
                }
                else {
                    int start = readBackToFirstSpecialChar(offset, "\n");
                    int end = getText(0, getLength()).indexOf("\n", offset);
                    if (start == -1) {
                        start = 0;
                    }
                    if (end == -1 || start == end) {
                        end = getLength();
                    }
                    HyperPanel.sourceDocumentLength = getLength();
                    UndoHandler.sourceOffset = offset;
                    insertTokenizer(start, getText(0, getLength()).substring(start, end));
                }
            }
        } catch (Exception ex) {}
    }

    public void remove(int offset, int length) {
        try {
            super.remove(offset, length);
            updateSaveStatus();
            if (HyperPanel.ch.getHighlight()) {
                int start = readBackToFirstSpecialChar(offset-1, "\n");
                int end = getText(0, getLength()).indexOf("\n", offset);
                if (start == -1) {
                    start = 0;
                }
                if (end == -1 || start == end) {
                    end = getLength();
                }
                HyperPanel.sourceDocumentLength = getLength();
                UndoHandler.sourceOffset = offset;
                insertTokenizer(start, getText(0, getLength()).substring(start, end));
            }
        } catch (Exception ex) {}
    }

    private void insertTokenizer(int offset, String text) {
        boolean isString = false;
        boolean isComment = false;
        int stringStart = 0;
        ProgressThread pth = null;
        StringTokenizer t = new StringTokenizer(text, hdoc.getDelimiters(), true);
        while (t.hasMoreElements()) {
            int token_number = t.countTokens();
            if (isLongInsert) {
                HyperPanel.jLabel.setText("Highlighting...");
                pth = new ProgressThread("Highlighting inserted text");
                pth.run();
                HyperPanel.prgBar.setMaximum(token_number);
            }
            for (int i=0; i < token_number; i++) {
                String token = t.nextToken();
                if (isLongInsert) {
                    HyperPanel.prgBar.setValue(i+1);
                }
                if (token.equals("\n")) {
                    isString = false;
                    isComment = false;
                }
                if (token.equals("\"") && !isComment) {
                    if (isString) {
                        isString = false;
                    } else {
                        isString = true;
                        stringStart = offset;
                    }
                    offset += token.length();
                }
                else if ((token.equals("#") || isComment) && !isString) {
                    isComment = true;
                    if (!isString) {
                        makeComment(offset, token.length(), null);
                    }
                    offset += token.length();
                }
                else {
                    if (!isString) {
                        evaluateWord(offset, token);
                    }
                    offset += token.length();
                }
                if (isString && !isComment) {
                    makeString(stringStart, offset-stringStart+1, null);
                }
            }
            if (isLongInsert) {
                pth.kill();
                HyperPanel.jLabel.setText("Ready");
                isLongInsert = false;
            }
        }
    }

    private SimpleAttributeSet getAttributeSet(AttributeSet attr) {
        SimpleAttributeSet sas;
        if (attr != null) {
            sas = new SimpleAttributeSet(attr);
        }
        else {
            sas = new SimpleAttributeSet();
        }
        return sas;
    }

    private void setAttributes(int offset, int length, SimpleAttributeSet sas) {
        setCharacterAttributes(offset, length, sas, false);
    }

    private void updateSaveStatus() {
        if (HyperPanel.sourceDocSaved) {
            HyperPanel.sourceDocSaved = false;
            HyperPanel.updateSaveButton();
        }
    }

    private void makeKeyword(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.keywordFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.keywordColor));
        setAttributes(offset, length, sas);
    }

    private void makeString(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.stringFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.stringColor));
        setAttributes(offset, length, sas);
    }

    private void makeNormal(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.generalFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.generalColor));
        setAttributes(offset, length, sas);
    }

    private void makeComment(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.commentFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.commentColor));
        setAttributes(offset, length, sas);
    }

    private void makeNumeric(int offset, int length, AttributeSet attr) {
        SimpleAttributeSet sas = getAttributeSet(attr);
        StyleConstants.setFontFamily(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getName());
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getStyle() == Font.BOLD) {
            StyleConstants.setBold(sas, true);
        }
        else {
            StyleConstants.setBold(sas, false);
        }
        if (((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getStyle() == Font.ITALIC) {
            StyleConstants.setItalic(sas, true);
        }
        else {
            StyleConstants.setItalic(sas, false);
        }
        StyleConstants.setFontSize(sas, ((Font)HyperPanel.confcache.get(HyperPanel.confcache.numericFont)).getSize());
        StyleConstants.setForeground(sas, (Color)HyperPanel.confcache.get(HyperPanel.confcache.numericColor));
        setAttributes(offset, length, sas);
    }

    private int readBackToFirstSpecialChar(int offset, String specialChar) {
        int firstSpecialChar = 0;
        String prevChar = "";
        try {
            if (offset > 0) {
                prevChar = getText(0, getLength()).substring(offset, offset+1);
            }
        } catch (BadLocationException blex) {
            blex.printStackTrace();
        }
        if (specialChar == null) {
            for (int i=0; i<hdoc.getSeparators().size(); i++) {
                if (prevChar.equals((String)hdoc.getSeparators().elementAt(i))) {
                    firstSpecialChar = offset;
                }
            }
        }
        else if (specialChar.equals(prevChar)) {
            firstSpecialChar = offset;
        }
        if (firstSpecialChar != 0) {
            return firstSpecialChar;
        }
        else if (offset > 0) {
            return readBackToFirstSpecialChar(offset-1, specialChar);
        }
        else {
            return 0;
        }
    }

    private void evaluateWord(int offset, String word) {
        if (hdoc.getKeywords().contains(word.toLowerCase())) {
            makeKeyword(offset, word.length(), null);
        }
        else if (hdoc.isNumeric(word)) {
            makeNumeric(offset, word.length(), null);
        }
        else {
            makeNormal(offset, word.length(), null);
        }
    }
}
