/*
 * $COPYRIGHT$
 * $Id: MyResources.java,v 1.3 2001/07/08 17:14:03 borzy Exp $
 *
 * Date        Author               Changes
 * Apr 18 2001 Borsos Szabolcs      Created
 */       
package org.media.hyperpad;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.InputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.PropertyResourceBundle;
import javax.swing.ImageIcon;
/**
 * Class meant to return results from the Resource Bundle to HyperPanel.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.3 $ $Date: 2001/07/08 17:14:03 $
 * @see String
 */          
public class MyResources extends PropertyResourceBundle {

    private static  MyResources _myRes = null;
    
    static {
        try{
           _myRes = new  MyResources("org/media/hyperpad/res/hyperpad.properties");
        } catch (IOException ioex) {
            ioex.printStackTrace();
        }
    }

    public MyResources(String property_file) throws IOException {
        super(getResourceAsStream(property_file));
    }

    public static String getLabels(String name) {
        String all = _myRes.getString(name);
        if (all.indexOf('-') != -1) {
            return all.substring(0, all.indexOf('-'));
        }
        else {
            return all;
        }
    }

    public static char getMnemonic(String name) {
        String all = _myRes.getString(name);
        char mnemonic = '!';
        String[] keys = tokenize(all);
        if (keys.length > 1) {
            mnemonic = keys[1].charAt(0);
        }
        return mnemonic;
    }

    public static char getShortcut(String name) {
        char shortcut = '!';
        try {
            String all = _myRes.getString(name);
            String[] keys = tokenize(all);
            if (keys.length > 2) {
                shortcut = keys[2].charAt(0);
            }
        } catch (Exception ex) {
            shortcut = '!';
        }
        return shortcut;
    }

    public static String getModifiers(String name) {
        String modifiers = "!";
        try {
            String all = _myRes.getString(name);
            String[] keys = tokenize(all);
            if (keys.length > 3) {
                modifiers = keys[3];
            }
        } catch (Exception ex) {
            modifiers = "!";
        }
        return modifiers;
    }

    public static String getTextValue(String name) {
        return _myRes.getString(name);
    }

    public static ImageIcon getImageIcon(String name) {
        String imgpath = _myRes.getString(name);
        Image img = null;
        try { 
            Class c = Class.forName("org.media.hyperpad.MyResources");
            InputStream in = c.getResourceAsStream(imgpath); 
            if (in == null) { 
                System.err.println("Image not found."); 
                return null; 
            } 
            byte[] buffer = new byte[in.available()]; 
            if (buffer.length == 1){
                int length = 0;
                while (in.read() != -1) { 
                    length++;
                }
                buffer = new byte[length];
                in.close();
                in = ClassLoader.getSystemResourceAsStream(imgpath);
            }
            in.read(buffer);
            img = Toolkit.getDefaultToolkit().createImage(buffer); 
            in.close();
        } 
        catch (Exception e) { 
            System.err.println("Unable to read image."); 
            e.printStackTrace(); 
            return null;
        }
        return new ImageIcon(img);
    }

    private static String[] tokenize(String input) {
        Vector v = new Vector();
        StringTokenizer t = new StringTokenizer(input, "-");
        String keys[];
        
        while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
        keys = new String[v.size()];
        for (int i = 0; i < keys.length; i++) {
            keys[i] = (String) v.elementAt(i);
        }
        return keys;
    }

    private static InputStream getResourceAsStream (String resource) throws IOException {
        InputStream i = ClassLoader.getSystemResourceAsStream(resource);        
        if (i != null) {
            return i;
        }
        else {                
            System.out.println("Can't find the resourcebundle properties file! " +
                               "It must be in a res/ directory in your CLASSPATH.");
            return null;
        }    
    }
}
