/*
 * $COPYRIGHT$
 * $Id: OpenRecentListHandler.java,v 1.4 2002/03/25 17:58:05 neuro Exp $
 *
 * Date        Author               Changes
 * Sep 06 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;
import org.media.hyperpad.conf.ConfigHandler;
/**
 * Class meant to handle the open recent list from file menu for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.4 $ $Date: 2002/03/25 17:58:05 $
 * @see String
 */  
public class OpenRecentListHandler {
    Document document;
    DocumentBuilderFactory factory;
    InputSource is;
    Element e;
        
    public OpenRecentListHandler() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            InputStream ins = getXML(getXMLPath());
            if (ins != null) {
                is = new InputSource(ins);
                DocumentBuilder builder = factory.newDocumentBuilder();
                document = builder.parse(is); 
                e = document.getDocumentElement();
            }
        }
        catch (Exception ex) {
            createXML(getXMLPath());
        }
    }
    
    private InputStream getXML(String path) throws IOException {
        InputStream ins = (InputStream)new FileInputStream(path);
        if (ins != null) {
            return ins;
        }
        else {
            return null;
        }
    }

    protected void appendList(String newItem) {
        java.util.Vector listItems = new java.util.Vector();
        for (int i = 0; i < 20; i++) {
            if (!getListItem(i).equals("")) {
                listItems.add(getListItem(i));
            }
        }
        if (listItems.contains(newItem)) {
            listItems.remove(newItem);
        }
        listItems.insertElementAt(newItem, 0);
        for (int i = 0; i < listItems.size(); i++) {
            setListItem(i, (String)listItems.get(i));
        }
        refreshRecentList();
    }

    public String getListItem(int itemNR) {
        return getItem("item"+itemNR, "value");
    }

    public void setListItem(int itemNR, String new_value) {
        setItem("item"+itemNR, "value", new_value);
    }

    public void refreshList() {
        refreshRecentList();
    }

    public void save() {
        saveXML(getXMLPath());
    }

    private String getItem(String node, String element) {
        String value = null;
        NodeList nl = e.getElementsByTagName(node);
        Element elem = (Element) nl.item(0);
        value = elem.getAttribute(element);
        return value;
    }

    private void setItem(String node, String element, String new_value) {
        NodeList nl = e.getElementsByTagName(node);
        Element elem = (Element) nl.item(0);
        elem.removeAttribute(element);
        elem.setAttribute(element, new_value);
    }


    private String getXMLPath() {
        File f = new File( System.getProperty("user.home") + HyperPanel.openRecentListFileName);
        return f.toString();
    }


    private void createXML(String path) {
        try {
            factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.newDocument();
            e = document.createElement("OpenRecentList");
            document.appendChild(e);
            for (int i = 0; i < 20; i++) {
                e.appendChild(document.createElement("item" + i));
                NodeList nl = e.getElementsByTagName("item" + i);
                Element elem = (Element) nl.item(0);
                elem.setAttribute("value", "");
            }
            saveXML(path);
        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    private void refreshRecentList() {
        ConfigHandler confHandler = new ConfigHandler(HyperPanel.XMLpath);
        int listLength = Integer.valueOf(confHandler.getRecentListLength()).intValue();
        for (int i = listLength; i < 20; i++) {
            NodeList nl = e.getElementsByTagName("item" + i);
            Element elem = (Element) nl.item(0);
            elem.setAttribute("value", "");
        }
        saveXML(getXMLPath());
    }

    private void saveXML(String path) {
        try {
            PrintWriter out = new PrintWriter(new OutputStreamWriter (new FileOutputStream(path)));
            org.apache.xml.serialize.OutputFormat outForm = 
                new org.apache.xml.serialize.OutputFormat(document);
            org.apache.xml.serialize.XMLSerializer serializer = 
                new org.apache.xml.serialize.XMLSerializer(out, outForm);
            outForm.setIndenting(true);
            serializer.serialize(e);
        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }
}
