/*
 * $COPYRIGHT$
 * $Id: PreferencesBox.java,v 1.40 2002/01/21 13:01:48 borzy Exp $
 *
 * Date        Author               Changes
 * Apr 18 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad;

import java.awt.Font;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import javax.swing.Box;
import javax.swing.JTree;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JScrollPane;
import javax.swing.JFileChooser;
import javax.swing.BorderFactory;
import javax.swing.JColorChooser;
import javax.swing.SwingConstants;
import javax.swing.text.html.StyleSheet;
import javax.swing.text.html.HTMLDocument;
import javax.swing.border.TitledBorder;
import javax.swing.border.CompoundBorder;
import org.media.hyperpad.conf.ConfigHandler;
import org.media.hyperpad.gui.*;
/**
 * Class meant to construct the PreferencesBox for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.40 $ $Date: 2002/01/21 13:01:48 $
 * @see String
 */  
public class PreferencesBox extends JDialog {
    JFrame frame = null;
    final String test_text = "This is a test text.";
    final HypTabbedPane tabPane = new HypTabbedPane(HypTabbedPane.TOP);
    HypLabel prevlabel;
    JPanel textpanel;
    private HypTextField browserPath, httpTF, ftpTF, httpPortTF, ftpPortTF;
    private ConfigHandler ch = new ConfigHandler(HyperPanel.XMLpath);
    private static String _olditem = "General";
    private static String _fontstyle = null;
    private final JPanel _bgcolorview = new JPanel();
    private final HypComboBox _itemcombo = new HypComboBox();
    private final HypComboBox _familycombo = new HypComboBox();
    private final HypComboBox _sizecombo = new HypComboBox();
    private HypRadioButton _plain = new HypRadioButton("Plain");
    private HypRadioButton _bold = new HypRadioButton("Bold");
    private HypRadioButton _italic = new HypRadioButton("Italic");
    private final JPanel _colorview = new JPanel();
    private HypButton resetButton;
    private final String[] itemnames = {"General", "Comment", "Keyword", "String", "Numeric", "Character"};
    private final String buttons = "reset ok apply cancel";

    public PreferencesBox(JFrame f) {
        super(f, "", true);
        this.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent ke) {
                if(ke.getKeyCode() == KeyEvent.VK_ESCAPE){
                    close();
                }
            }
        });
        frame = f;
        tabPane.setFont(ch.getMenuFont());
        tabPane.setRequestFocusEnabled(false);
        tabPane.addTab("General", createGeneralPanel());
        tabPane.addTab(HyperPanel.getResourceString("navigationLabel"), createNavPanel());
        tabPane.addTab(HyperPanel.getResourceString("editingLabel"), createEditingPanel());
        tabPane.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
                if (tabPane.getSelectedIndex() == 2) {
                    resetButton.setEnabled(true);
                }
                else {
                    resetButton.setEnabled(false);
                }
            }
        });
        tabPane.addMouseMotionListener(new MouseMotionListener() {
            public void mouseDragged(MouseEvent me) {}     
            public void mouseMoved(MouseEvent me) {
                int tabIndex = tabPane.getUI().tabForCoordinate(tabPane, me.getX(), me.getY());
                HyperPanel.setTabForegroundAt(tabPane, tabIndex);
            }
        });
        tabPane.setSelectedIndex(0);
        JPanel bpanel = new JPanel(new BorderLayout());
        bpanel.setBorder(BorderFactory.createEmptyBorder(6,0,0,0));
        bpanel.add(buttonPanel(), BorderLayout.EAST);
        JPanel main = new JPanel(new BorderLayout());
        main.setBorder(BorderFactory.createEmptyBorder(12,6,11,6));
        main.add(tabPane, BorderLayout.CENTER);
        main.add(bpanel, BorderLayout.SOUTH);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(main, BorderLayout.CENTER);
        
        setResizable(true);
        setTitle(HyperPanel.getResourceString("PrefBoxWindowLabel"));
        setModal(true);
        pack();
        setLocationRelativeTo(f);
    }

    private JPanel buttonPanel() {
        String[] buttonKeys = HyperPanel.tokenize(buttons);
        JPanel panel = new JPanel(new GridLayout(1,buttonKeys.length,5,0));
        for (int i = 0; i < buttonKeys.length; i++) {
            if (buttonKeys[i].equals("reset")) {
                resetButton = new HypButton(MyResources.getTextValue(buttonKeys[i] + "Label"));
                resetButton.setFont(ch.getMenuFont());
                HyperPanel.cm.addComponent(resetButton);
                resetButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent ae) {
                        if (tabPane.getSelectedIndex() == 2) {
                            resetToDefaults();
                        }
                    }
                });
                resetButton.setEnabled(false);
                resetButton.setRequestFocusEnabled(false);
                panel.add(resetButton);
            }
            else {
                HypButton button = new HypButton(MyResources.getTextValue(buttonKeys[i] + "Label"));
                button.setFont(ch.getMenuFont());
                HyperPanel.cm.addComponent(button);
                if (buttonKeys[i].equals("cancel")) {
                    button.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent ae) {
                            itemchanger("General");
                            close();
                        }
                    });
                }
                else if (buttonKeys[i].equals("ok")) {
                    button.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent ae) {
                            save();
                            HyperPanel.orlh.refreshList();
                            HyperPanel.updateOpenRecentMenu();
                            itemchanger("General");
                            close();
                        }
                    });
                }
                else if (buttonKeys[i].equals("apply")) {
                    button.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent ae) {
                            save();
                        }
                    });
                }
                button.setRequestFocusEnabled(false);
                panel.add(button);
            }
        }
        return panel;
    }

    private JPanel createGeneralPanel() {
        JPanel general = new JPanel(new BorderLayout());
        final HypLabel tab = new HypLabel(HyperPanel.getResourceString("tabWidthLabel"));
        tab.setFont(ch.getMenuFont());
        tab.setForeground(Color.black);
        final HypTextField spaceNR = new HypTextField(3);
        spaceNR.setText(ch.getSpaceNumber());
        spaceNR.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent ke) {
                if (spaceNR.getText().length() != 0) {
                    try {
                        int test = Integer.valueOf(spaceNR.getText()).intValue();
                        ch.setSpaceNumber(spaceNR.getText());
                    } catch (NumberFormatException nfex) {
                        spaceNR.setText("0");
                    }
                }
            }
        });
        final HypCheckBox replace = new HypCheckBox(HyperPanel.getResourceString("tabToSpaceLabel"));
        replace.setFont(ch.getMenuFont());
        replace.setForeground(Color.black);
        replace.setRequestFocusEnabled(false);
        replace.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (replace.isSelected()) {
                    tab.setEnabled(true);
                    spaceNR.setEnabled(true);
                    ch.setReplaceMode(true);
                }
                else if (!replace.isSelected()) {
                    tab.setEnabled(false);
                    spaceNR.setEnabled(false);
                    ch.setReplaceMode(false);
                }
            }
        });
        if (ch.getReplaceMode()) {
            replace.setSelected(true);
            tab.setEnabled(true);
            spaceNR.setEnabled(true);
        }
        else {
              replace.setSelected(false);
              tab.setEnabled(false);
              spaceNR.setEnabled(false);
        }
        final HypCheckBox antialiasing = new HypCheckBox(HyperPanel.getResourceString("useAntiAliasingLabel"));
        antialiasing.setFont(ch.getMenuFont());
        antialiasing.setForeground(Color.black);
        antialiasing.setRequestFocusEnabled(false);
        antialiasing.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (antialiasing.isSelected()) {
                    ch.setAntiAliasingStatus(true);
                }
                else if (!antialiasing.isSelected()) {
                    ch.setAntiAliasingStatus(false);
                }
            }
        });
        if (ch.getAntiAliasingStatus()) {
            antialiasing.setSelected(true);
        }
        else {
            antialiasing.setSelected(false);
        }
        final HypCheckBox linewrap = new HypCheckBox(HyperPanel.getResourceString("useLineWrapLabel"));
        linewrap.setFont(ch.getMenuFont());
        linewrap.setForeground(Color.black);
        linewrap.setRequestFocusEnabled(false);
        linewrap.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (linewrap.isSelected()) {
                    ch.setLineWrapStatus(true);
                }
                else if (!linewrap.isSelected()) {
                    ch.setLineWrapStatus(false);
                }
            }
        });
        if (ch.getLineWrapStatus()) {
            linewrap.setSelected(true);
        }
        else {
            linewrap.setSelected(false);
        }
        HypLabel hist = new HypLabel(HyperPanel.getResourceString("cmdHistLabel"));
        hist.setFont(ch.getMenuFont());
        hist.setForeground(Color.black);
        final HypTextField histLength = new HypTextField(3);
        histLength.setText(ch.getHistoryLength());
        histLength.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent ke) {
                if (histLength.getText().length() != 0) {
                    try {
                        int test = Integer.valueOf(histLength.getText()).intValue();
                        ch.setHistoryLength(histLength.getText());
                    } catch (NumberFormatException nfex) {
                        histLength.setText("0");
                    }
                }
            }
        });
        HypLabel recent = new HypLabel(HyperPanel.getResourceString("recentLabel"));
        recent.setFont(ch.getMenuFont());
        recent.setForeground(Color.black);
        java.util.Vector valueList = new java.util.Vector(20);
        for (int i = 1; i < 21; i++) {
            valueList.add(""+i);
        }
        final HypComboBox recentListLength = new HypComboBox(valueList);
        recentListLength.setFont(ch.getMenuFont());
        recentListLength.setForeground(Color.black);
        recentListLength.setEditable(false);
        recentListLength.setSelectedItem(ch.getRecentListLength());
        recentListLength.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent ie) {
                ch.setRecentListLength(recentListLength.getSelectedItem().toString());
            }
        });
        JPanel labels = new JPanel(new GridLayout(3,1,0,12));
        labels.add(tab);
        labels.add(hist);
        labels.add(recent);
        JPanel fields = new JPanel(new GridLayout(3,1,0,3));
        fields.add(spaceNR);
        fields.add(histLength);
        fields.add(recentListLength);
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT,5,0));
        panel.add(labels);
        panel.add(fields);
        JPanel checkPanel = new JPanel(new GridLayout(3,1));
        checkPanel.add(antialiasing);
        checkPanel.add(linewrap);
        checkPanel.add(replace);
        checkPanel.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
        JPanel main = new JPanel();
        main.add(checkPanel);
        main.add(panel);
        main.setLayout(new javax.swing.BoxLayout(main, javax.swing.BoxLayout.Y_AXIS));
        main.setBorder(BorderFactory.createCompoundBorder(addBorder("General Setting", "etched"),
                                                          BorderFactory.createEmptyBorder(0,6,5,6)));
        general.add(main, BorderLayout.NORTH);
        general.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        return general;
    }

    private JPanel createNavPanel() {
        JPanel container = new JPanel(new BorderLayout());
        JPanel navigation = new JPanel(new BorderLayout());
        final HypCheckBox use = new HypCheckBox("Use Proxy");
        use.setFont(ch.getMenuFont());
        use.setForeground(Color.black);
        use.setRequestFocusEnabled(false);
        use.setSelected(true);
        HyperPanel.cm.addComponent(use);
        HypLabel http = new HypLabel("HTTP Proxy:");
        http.setFont(ch.getMenuFont());
        http.setForeground(Color.black);
        HyperPanel.cm.addProxyPanelComponent(http);
        HypLabel ftp = new HypLabel("FTP   Proxy:");
        ftp.setFont(ch.getMenuFont());
        ftp.setForeground(Color.black);
        HyperPanel.cm.addProxyPanelComponent(ftp);
        HypLabel httpPort = new HypLabel("Port:");
        httpPort.setFont(ch.getMenuFont());
        httpPort.setForeground(Color.black);
        HyperPanel.cm.addProxyPanelComponent(httpPort);
        HypLabel ftpPort = new HypLabel("Port:");
        ftpPort.setFont(ch.getMenuFont());
        ftpPort.setForeground(Color.black);
        HyperPanel.cm.addProxyPanelComponent(ftpPort);
        HypLabel browsPath = new HypLabel("Browser path:");
        browsPath.setFont(ch.getMenuFont());
        browsPath.setForeground(Color.black);
        browserPath = new HypTextField(ch.getBrowser(), 23);
        JPanel usePanel = new JPanel(new BorderLayout());
        usePanel.add(use, BorderLayout.WEST);
        use.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (use.isSelected()) {
                    HyperPanel.cm.updateProxyPanelComponents(true);
                    ch.setProxyStatus(true);
                    System.getProperties().put("proxySet", "true");
                    System.getProperties().put("proxyHost", ch.getHttpProxyHost());
                    System.getProperties().put("proxyPort", ch.getHttpProxyPort());
                    System.getProperties().put("ftpProxySet", "true");
                    System.getProperties().put("ftpProxyHost", ch.getFtpProxyHost());
                    System.getProperties().put("ftpProxyPort", ch.getFtpProxyPort());
                }
                else if (!use.isSelected()) {
                    HyperPanel.cm.updateProxyPanelComponents(false);
                    ch.setProxyStatus(false);
                    System.getProperties().put("proxySet", "false");
                    System.getProperties().put("ftpProxySet", "false");
                }
            }
        });
        JPanel browserANDusePanel = new JPanel(new GridLayout(3,1));
        HypButton changeBrowser = new HypButton(HyperPanel.getResourceString("changeLabel"));
        changeBrowser.setFont(ch.getMenuFont());
        changeBrowser.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actev) {
                JFileChooser chooser = new JFileChooser();
                int returnVal = chooser.showOpenDialog(frame);
                if(returnVal == JFileChooser.APPROVE_OPTION) {
                    String selectedPath = chooser.getSelectedFile().getPath();
                    browserPath.setText(selectedPath);
                    ch.setBrowser(selectedPath);
                }
            }
        });
        JPanel browserPanel = new JPanel(new BorderLayout());
        JPanel bPanel = new JPanel(new BorderLayout());
        bPanel.add(Box.createHorizontalStrut(5), BorderLayout.WEST);
        bPanel.add(changeBrowser, BorderLayout.CENTER);
        browserPanel.add(browsPath, BorderLayout.WEST);
        browserPanel.add(browserPath, BorderLayout.CENTER);
        browserPanel.add(bPanel, BorderLayout.EAST);
        JPanel emptyPanel = new JPanel();
        browserANDusePanel.add(browserPanel);
        browserANDusePanel.add(emptyPanel);
        browserANDusePanel.add(usePanel);
        JPanel labelPanel = new JPanel(new GridLayout(4,1,0,3));
        labelPanel.add(http);
        labelPanel.add(ftp);
        JPanel TFPanel = new JPanel(new GridLayout(4,1,0,3));
        JPanel httpTFPanel = new JPanel(new BorderLayout());
        httpTF = new HypTextField(ch.getHttpProxyHost(), 13);
        HyperPanel.cm.addProxyPanelComponent(httpTF);
        httpTFPanel.add(httpTF, BorderLayout.CENTER);
        JPanel httpPortPanel = new JPanel(new BorderLayout());
        httpPortTF = new HypTextField(ch.getHttpProxyPort(), 5);
        HyperPanel.cm.addProxyPanelComponent(httpPortTF);
        httpPortPanel.add(httpPort, BorderLayout.WEST);
        httpPortPanel.add(httpPortTF, BorderLayout.CENTER);
        httpTFPanel.add(httpPortPanel, BorderLayout.EAST);
        TFPanel.add(httpTFPanel);
        JPanel ftpTFPanel = new JPanel(new BorderLayout());
        ftpTF = new HypTextField(ch.getFtpProxyHost(), 13);
        HyperPanel.cm.addProxyPanelComponent(ftpTF);
        ftpTFPanel.add(ftpTF, BorderLayout.CENTER);
        JPanel ftpPortPanel = new JPanel(new BorderLayout());
        ftpPortTF = new HypTextField(ch.getFtpProxyPort(), 5);
        HyperPanel.cm.addProxyPanelComponent(ftpPortTF);
        ftpPortPanel.add(ftpPort, BorderLayout.WEST);
        ftpPortPanel.add(ftpPortTF, BorderLayout.CENTER);
        ftpTFPanel.add(ftpPortPanel, BorderLayout.EAST);
        TFPanel.add(ftpTFPanel);
        if (ch.getProxyStatus()) {
            use.setSelected(true);
            HyperPanel.cm.updateProxyPanelComponents(true);
        }
        else {
            use.setSelected(false);
            HyperPanel.cm.updateProxyPanelComponents(false);
        }
        navigation.add(browserANDusePanel, BorderLayout.NORTH);
        navigation.add(labelPanel, BorderLayout.WEST);
        navigation.add(TFPanel, BorderLayout.CENTER);
        navigation.setBorder(BorderFactory.createEmptyBorder(12,12,12,12));
        container.add(navigation, BorderLayout.NORTH);
        return container;
    }

    private JPanel createEditingPanel() {
        JPanel editing = new JPanel(new BorderLayout());
        editing.add(bgcolor(), BorderLayout.NORTH);
        editing.add(highlight(), BorderLayout.CENTER);
        editing.add(preview(), BorderLayout.SOUTH);
        editing.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        return editing;
    }

    private JPanel bgcolor() {
        JPanel bgcolor = new JPanel(new FlowLayout(FlowLayout.LEFT, 12, 0));
        _bgcolorview.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
        _bgcolorview.setPreferredSize(new Dimension(80,30));
        _bgcolorview.setBackground(ch.getGeneralBgColor());
        HypButton change = new HypButton(HyperPanel.getResourceString("changeLabel"));
        change.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(change);
        change.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                Color selected = JColorChooser.showDialog(frame, "Background Color Chooser", 
                                                          _bgcolorview.getBackground());
                if (selected != null) {
                    _bgcolorview.setBackground(selected);
                    colorchanged(selected, "background");
                }
            }
        });
        bgcolor.setBorder(BorderFactory.createCompoundBorder(addBorder("Background Color", "etched"),
                                                             BorderFactory.createEmptyBorder(0,0,5,0)));
        bgcolor.add(Box.createHorizontalStrut(37));
        bgcolor.add(_bgcolorview);
        bgcolor.add(change);
        return bgcolor;
    }

    private JPanel highlight() {
        JPanel highlight = new JPanel(new BorderLayout());
        highlight.setBorder(BorderFactory.createCompoundBorder(addBorder("Highlight", "etched"), 
                                                               BorderFactory.createEmptyBorder(5,6,5,6)));
        JPanel itempanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 12, 0));
        HypLabel itemlabel = new HypLabel("Item:");
        itemlabel.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(itemlabel);
        itemlabel.setForeground(Color.black);
        _itemcombo.setEditable(false);
        _itemcombo.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(_itemcombo);
        for (int i=0; i<itemnames.length; i++) {
            _itemcombo.addItem(itemnames[i]);
        }
        _itemcombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                HypComboBox cb = (HypComboBox)e.getSource();
                String sel_item = (String)cb.getSelectedItem();
                itemchanger(sel_item);
            }
        });
        itempanel.add(itemlabel);
        itempanel.add(_itemcombo);

        JPanel font = new JPanel(new BorderLayout());
        font.setBorder(addBorder("Font", "line"));
        _familycombo.setEditable(true);
        _familycombo.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(_familycombo);
        _familycombo.addItem(ch.getMenuFont().getName());
        String[] systemfonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        for (int i=0; i<systemfonts.length; i++) {
            if (systemfonts[i].equals(ch.getGeneralFont().getName())) {}
            else {
                _familycombo.addItem(systemfonts[i]);
            }
        }
        _familycombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                HypComboBox cb = (HypComboBox)e.getSource();
                String fontName = (String)cb.getSelectedItem();
                fontchanged(fontName, new Integer((String)_sizecombo.getSelectedItem()).intValue(), _fontstyle);
            }
        });
        _sizecombo.setEditable(false);
        _sizecombo.setPreferredSize(new Dimension(60,24));
        _sizecombo.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(_sizecombo);
        for (int i=6; i<25; i++) {
            if (i == ch.getGeneralFont().getSize()) {
                _sizecombo.addItem(""+i);
                _sizecombo.setSelectedItem(""+i);
            }
            else {
                _sizecombo.addItem(""+i);
            }
        }
        _sizecombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                HypComboBox cb = (HypComboBox)e.getSource();
                String boxName = (String)cb.getSelectedItem();
                fontchanged((String)_familycombo.getSelectedItem(), new Integer(boxName).intValue(), _fontstyle);
            }
        });
        JPanel combopanel = new JPanel();
        JPanel fcp = new JPanel();
        fcp.add(_familycombo);
        fcp.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(0,0,0,0), "Family:", TitledBorder.LEFT,
                                                       TitledBorder.TOP, ch.getMenuFont(), Color.black));
        combopanel.add(fcp);
        JPanel scp = new JPanel();
        scp.add(Box.createHorizontalStrut(30));
        scp.add(_sizecombo);
        scp.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(0,0,0,0), "Size:", TitledBorder.CENTER,
                                                       TitledBorder.TOP, ch.getMenuFont(), Color.black));
        combopanel.add(scp);
        ButtonGroup bg = new ButtonGroup();
        _plain.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(_plain);
        _plain.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fontchanged((String)_familycombo.getSelectedItem(), 
                            new Integer((String)_sizecombo.getSelectedItem()).intValue(), "plain");
                _fontstyle = "plain";
            }
        });
        _bold.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(_bold);
        _bold.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fontchanged((String)_familycombo.getSelectedItem(), 
                            new Integer((String)_sizecombo.getSelectedItem()).intValue(), "bold");
                _fontstyle = "bold";
            }
        });
        _italic.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(_italic);
        _italic.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fontchanged((String)_familycombo.getSelectedItem(), 
                            new Integer((String)_sizecombo.getSelectedItem()).intValue(), "italic");
                _fontstyle = "italic";
            }
        });
        bg.add(_plain);
        bg.add(_bold);
        bg.add(_italic);
        JPanel buttonpanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 20, 0));
        buttonpanel.add(_plain);
        buttonpanel.add(_bold);
        buttonpanel.add(_italic);
        setRadioButtons(ch.getMenuFont().getStyle());
        font.add(combopanel, BorderLayout.NORTH);
        font.add(buttonpanel, BorderLayout.SOUTH);

        JPanel color = new JPanel(new FlowLayout(FlowLayout.LEFT, 12, 0));
        HypLabel colorlabel = new HypLabel("Color:");
        colorlabel.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(colorlabel);
        colorlabel.setForeground(Color.black);
        _colorview.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
        _colorview.setPreferredSize(new Dimension(80,30));
        _colorview.setBackground(ch.getGeneralColor());
        HypButton change = new HypButton(HyperPanel.getResourceString("changeLabel"));
        change.setFont(ch.getMenuFont());
        HyperPanel.cm.addComponent(change);
        change.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                Color selected = JColorChooser.showDialog(frame, "Font Color Chooser", _colorview.getBackground());
                if (selected != null) {
                    _colorview.setBackground(selected);
                    colorchanged(selected, "font");
                }
            }
        });
        color.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
        color.add(colorlabel);
        color.add(_colorview);
        color.add(change);

        highlight.add(itempanel, BorderLayout.NORTH);
        highlight.add(font, BorderLayout.CENTER);
        highlight.add(color, BorderLayout.SOUTH);
        return highlight;
    }

    private JPanel preview() {
        JPanel preview = new JPanel(new BorderLayout());
        prevlabel = new HypLabel(test_text, SwingConstants.CENTER);
        textpanel = new JPanel(new BorderLayout());
        textpanel.setBackground(ch.getGeneralBgColor());
        prevlabel.setForeground(ch.getGeneralColor());
        prevlabel.setFont(ch.getMenuFont());
        textpanel.add(prevlabel, BorderLayout.CENTER);
        textpanel.setBorder(BorderFactory.createLoweredBevelBorder());
        textpanel.setPreferredSize(new Dimension(100,30));
        preview.add(textpanel, BorderLayout.CENTER);
        preview.setBorder(BorderFactory.createCompoundBorder(addBorder("Preview", "etched"), 
                                                             BorderFactory.createEmptyBorder(0,6,5,6)));
        return preview;
    }

    private TitledBorder addBorder(String title, String type) {
        TitledBorder tb = null;
        if (type.equals("etched")) {
            tb = BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), title, 
                                                  TitledBorder.LEFT, TitledBorder.TOP, 
                                                  ch.getMenuFont(), Color.black);
        }
        else if (type.equals("line")) {
            tb = BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray, 1), title, 
                                                  TitledBorder.LEFT, TitledBorder.TOP, 
                                                  ch.getMenuFont(), Color.black);
        }
        return tb;
    }

    private void fontchanged(String family, int size, String type) {
        if (type.equals("plain")) {
            prevlabel.setFont(new Font(family, Font.PLAIN, size));
        }
        else if (type.equals("bold")) {
            prevlabel.setFont(new Font(family, Font.BOLD, size));
        }
        else if (type.equals("italic")) {
            prevlabel.setFont(new Font(family, Font.ITALIC, size));
        }
    }

    private void colorchanged(Color newcolor, String type) {
        if (type.equals("font")) {
            prevlabel.setForeground(newcolor);
        }
        else if (type.equals("background")) {
            textpanel.setBackground(newcolor);
        }
    }

    private void itemchanger(String new_item) {
        if (_olditem.equals("General")) {
            Font f = prevlabel.getFont();
            ch.setGeneralFont(f.getName(), f.getStyle(), f.getSize());
            Color c = prevlabel.getForeground();
            ch.setGeneralColor(c.getRed(), c.getGreen(), c.getBlue());
            Color bgc = textpanel.getBackground();
            ch.setGeneralBgColor(bgc.getRed(), bgc.getGreen(), bgc.getBlue());
            _olditem = new_item;
        }
        else if (_olditem.equals("Comment")) {
            Font f = prevlabel.getFont();
            ch.setCommentFont(f.getName(), f.getStyle(), f.getSize());
            Color c = prevlabel.getForeground();
            ch.setCommentColor(c.getRed(), c.getGreen(), c.getBlue());
            _olditem = new_item;
        }
        else if (_olditem.equals("Keyword")) {
            Font f = prevlabel.getFont();
            ch.setKeywordFont(f.getName(), f.getStyle(), f.getSize());
            Color c = prevlabel.getForeground();
            ch.setKeywordColor(c.getRed(), c.getGreen(), c.getBlue());
            _olditem = new_item;
        }
        else if (_olditem.equals("String")) {
            Font f = prevlabel.getFont();
            ch.setStringFont(f.getName(), f.getStyle(), f.getSize());
            Color c = prevlabel.getForeground();
            ch.setStringColor(c.getRed(), c.getGreen(), c.getBlue());
            _olditem = new_item;
        }
        else if (_olditem.equals("Numeric")) {
            Font f = prevlabel.getFont();
            ch.setNumericFont(f.getName(), f.getStyle(), f.getSize());
            Color c = prevlabel.getForeground();
            ch.setNumericColor(c.getRed(), c.getGreen(), c.getBlue());
            _olditem = new_item;
        }
        else if (_olditem.equals("Character")) {
            Font f = prevlabel.getFont();
            ch.setCharacterFont(f.getName(), f.getStyle(), f.getSize());
            Color c = prevlabel.getForeground();
            ch.setCharacterColor(c.getRed(), c.getGreen(), c.getBlue());
            _olditem = new_item;
        }
        if (new_item.equals("General")) {
            prevlabel.setFont(ch.getGeneralFont());
            prevlabel.setForeground(ch.getGeneralColor());
            prevlabel.setBackground(ch.getGeneralBgColor());
            setRadioButtons(ch.getGeneralFont().getStyle());
            _sizecombo.setSelectedItem(""+ch.getGeneralFont().getSize());
            _familycombo.setSelectedItem(ch.getGeneralFont().getName());
            _bgcolorview.setBackground(ch.getGeneralBgColor());
            _colorview.setBackground(ch.getGeneralColor());
        }
        else if (new_item.equals("Comment")) {
            prevlabel.setFont(ch.getCommentFont());
            prevlabel.setForeground(ch.getCommentColor());
            setRadioButtons(ch.getCommentFont().getStyle());
            _sizecombo.setSelectedItem(""+ch.getCommentFont().getSize());
            _familycombo.setSelectedItem(ch.getCommentFont().getName());
            _colorview.setBackground(ch.getCommentColor());
        }
        else if (new_item.equals("Keyword")) {
            prevlabel.setFont(ch.getKeywordFont());
            prevlabel.setForeground(ch.getKeywordColor());
            setRadioButtons(ch.getKeywordFont().getStyle());
            _sizecombo.setSelectedItem(""+ch.getKeywordFont().getSize());
            _familycombo.setSelectedItem(ch.getKeywordFont().getName());
            _colorview.setBackground(ch.getKeywordColor());
        }
        else if (new_item.equals("String")) {
            prevlabel.setFont(ch.getStringFont());
            prevlabel.setForeground(ch.getStringColor());
            setRadioButtons(ch.getStringFont().getStyle());
            _sizecombo.setSelectedItem(""+ch.getStringFont().getSize());
            _familycombo.setSelectedItem(ch.getStringFont().getName());
            _colorview.setBackground(ch.getStringColor());
        }
        else if (new_item.equals("Numeric")) {
            prevlabel.setFont(ch.getNumericFont());
            prevlabel.setForeground(ch.getNumericColor());
            setRadioButtons(ch.getNumericFont().getStyle());
            _sizecombo.setSelectedItem(""+ch.getNumericFont().getSize());
            _familycombo.setSelectedItem(ch.getNumericFont().getName());
            _colorview.setBackground(ch.getNumericColor());
        }
        else if (new_item.equals("Character")) {
            prevlabel.setFont(ch.getCharacterFont());
            prevlabel.setForeground(ch.getCharacterColor());
            setRadioButtons(ch.getCharacterFont().getStyle());
            _sizecombo.setSelectedItem(""+ch.getCharacterFont().getSize());
            _familycombo.setSelectedItem(ch.getCharacterFont().getName());
            _colorview.setBackground(ch.getCharacterColor());
        }
    }

    private void setRadioButtons(int select) {
        if (select == Font.PLAIN) {
            _plain.setSelected(true);
            _fontstyle = "plain";
        }
        else if (select == Font.BOLD) {
            _bold.setSelected(true);
            _fontstyle = "bold";
        }
        else if (select == Font.ITALIC) {
            _italic.setSelected(true);
            _fontstyle = "italic";
        }
    }

    private void resetToDefaults() {
        ch.setGeneralFont("Dialog", Font.PLAIN, 12);
        ch.setGeneralColor(0,0,0);
        ch.setGeneralBgColor(255,255,255);
        ch.setCommentFont("Dialog", Font.PLAIN, 12);
        ch.setCommentColor(0,0,0);
        ch.setKeywordFont("Dialog", Font.PLAIN, 12);
        ch.setKeywordColor(0,0,0);
        ch.setStringFont("Dialog", Font.PLAIN, 12);
        ch.setStringColor(0,0,0);
        ch.setNumericFont("Dialog", Font.PLAIN, 12);
        ch.setNumericColor(0,0,0);
        ch.setCharacterFont("Dialog", Font.PLAIN, 12);
        ch.setCharacterColor(0,0,0);
        prevlabel.setFont(ch.getMenuFont());
        prevlabel.setForeground(ch.getGeneralColor());
        textpanel.setBackground(ch.getGeneralBgColor());
        prevlabel.setBackground(ch.getGeneralBgColor());
        setRadioButtons(ch.getMenuFont().getStyle());
        _sizecombo.setSelectedItem(""+ch.getMenuFont().getSize());
        _familycombo.setSelectedItem(ch.getMenuFont().getName());
        _bgcolorview.setBackground(ch.getGeneralBgColor());
        _colorview.setBackground(ch.getGeneralColor());
    }

    private void save() {
        itemchanger(_olditem);
        ch.setBrowser(browserPath.getText().trim());
        ch.setHttpProxyHost(httpTF.getText().trim());
        ch.setHttpProxyPort(httpPortTF.getText().trim());
        ch.setFtpProxyHost(ftpTF.getText().trim());
        ch.setFtpProxyPort(ftpPortTF.getText().trim());
        ch.save(HyperPanel.XMLpath);
        HyperPanel.confcache.inicialize();
        HyperPanel.ch = ch;
    }

    private void close() { 
        this.dispose();
    }
}
