/*
 * $COPYRIGHT$
 * $Id: ConfigHandler.java,v 1.25 2002/01/21 13:00:38 borzy Exp $
 *
 * Date        Author               Changes
 * Apr 18 2001 Borsos Szabolcs      Created
 */   
package org.media.hyperpad.conf;

import java.awt.Font;
import java.awt.Color;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;
/**
 * Class meant to handle the configuration XML for HyperPad.
 * @author <a href="mailto:borzy@nolimits.ro">Borsos Szabolcs</a>
 * @version $Revision: 1.25 $ $Date: 2002/01/21 13:00:38 $
 * @see String
 */  
public class ConfigHandler {
    Document document;
    InputSource is;
    Element e;
        
    public ConfigHandler(String path) {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            InputStream ins = getXML(path);
            if (ins != null) {
                is = new InputSource(ins);
                DocumentBuilder builder = factory.newDocumentBuilder();
                document = builder.parse(is); 
                e = document.getDocumentElement();
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    private InputStream getXML(String path) throws IOException {
        try {
            Class _this = Class.forName("org.media.hyperpad.conf.ConfigHandler");
            InputStream ins = (InputStream)new FileInputStream(path);
            if (ins != null) {
                return ins;
            }
            else {
                System.out.println("Wrong path given to ConfigHandler. "+path);
                return null;
            }
        }
        catch (ClassNotFoundException cnfex) {
            System.out.println("Can't find the configuration XML file! Loading default values...");
            cnfex.printStackTrace();
            return null;
        }
    }

    public boolean getNavigationBarStatus() {
        return getMode("NavigationBarStatus", "", "status");
    }

    public void setNavigationBarStatus(boolean new_status) {
        setMode("NavigationBarStatus", "", "status", new_status);
    }

    public String getButtonDisplay() {
        if (getFirstChildValue("NavigationBarStatus", "show").toLowerCase().trim().equals("text only")) {
            return "TO";
        }
        else if (getFirstChildValue("NavigationBarStatus", "show").toLowerCase().trim().equals("text and icons")) {
            return "T+I";
        }
        else if (getFirstChildValue("NavigationBarStatus", "show").toLowerCase().trim().equals("icons only")) {
            return "IO";
        }
        else {
            System.out.println("Wrong Navigation Button show value entered. Please correct the errors " +
                               "in this file: conf/hyperpad.xml. Continue loading" +
                               "with default setting: show text and icons.");
            return "T+I";
        }
    }

    public void setButtonDisplay(String new_display) {
        if (new_display.equals("TO")) {
            setFirstChildValue("NavigationBarStatus", "show", "text only");
        }
        else if (new_display.equals("T+I")) {
            setFirstChildValue("NavigationBarStatus", "show", "text and icons");
        }
        else if (new_display.equals("IO")) {
            setFirstChildValue("NavigationBarStatus", "show", "icons only");
        }
    }

    public boolean getReplaceMode() {
        return getMode("TabWidth", "", "replace");
    }
    
    public void setReplaceMode(boolean new_mode) {
        setMode("TabWidth", "", "replace", new_mode);
    }

    public String getSpaceNumber() {
        return getFirstChildValue("TabWidth", "spaceNR");
    }
    
    public void setSpaceNumber(String number) {
        setFirstChildValue("TabWidth", "spaceNR", number);
    }

    public String getHistoryLength() {
        return getFirstChildValue("HistoryLength", "length");
    }

    public void setHistoryLength(String length) {
        setFirstChildValue("HistoryLength", "length", length);
    }

    public String getRecentListLength() {
        return getFirstChildValue("RecentListLength", "length");
    }

    public void setRecentListLength(String length) {
        setFirstChildValue("RecentListLength", "length", length);
    }

    public boolean getHTMLMode() {
        return getMode("HTMLMode", "", "mode");
    }

    public void setHTMLMode(boolean new_mode) {
        setMode("HTMLMode", "", "mode", new_mode);
    }

    public boolean getHighlight() {
        return getMode("Highlight", "", "mode");
    }

    public void setHighlight(boolean new_mode) {
        setMode("Highlight", "", "mode", new_mode);
    }

    public String getLookAndFeel() {
        return getFirstChildValue("LookAndFeel", "current");
    }

    public void setLookAndFeel(String new_LF) {
        setFirstChildValue("LookAndFeel", "current", new_LF);
    }

    public boolean getAntiAliasingStatus() {
        return getMode("AntiAliasing", "", "on");
    }

    public void setAntiAliasingStatus(boolean new_status) {
        setMode("AntiAliasing", "", "on", new_status);
    }

    public boolean getLineWrapStatus() {
        return getMode("LineWrap", "", "on");
    }

    public void setLineWrapStatus(boolean new_status) {
        setMode("LineWrap", "", "on", new_status);
    }

    public String getBrowser() {
        return getSecondChildValue("navigation", "Browser", "path");
    }

    public void setBrowser(String new_path) {
        setSecondChildValue("navigation", "Browser", "path", new_path);
    }
    
    public boolean getProxyStatus() {
        return getMode("navigation", "Proxy", "enabled");
    }

    public void setProxyStatus(boolean new_status) {
        setMode("navigation", "Proxy", "enabled", new_status);
    }

    public String getHttpProxyHost() {
        return getSecondChildValue("navigation", "HttpProxy", "host");
    }

    public void setHttpProxyHost(String new_value) {
        setSecondChildValue("navigation", "HttpProxy", "host", new_value);
    }

    public String getHttpProxyPort() {
        return getSecondChildValue("navigation", "HttpProxy", "port");
    }

    public void setHttpProxyPort(String new_value) {
        setSecondChildValue("navigation", "HttpProxy", "port", new_value);
    }

    public String getFtpProxyHost() {
        return getSecondChildValue("navigation", "FtpProxy", "host");
    }

    public void setFtpProxyHost(String new_value) {
        setSecondChildValue("navigation", "FtpProxy", "host", new_value);
    }

    public String getFtpProxyPort() {
        return getSecondChildValue("navigation", "FtpProxy", "port");
    }

    public void setFtpProxyPort(String new_value) {
        setSecondChildValue("navigation", "FtpProxy", "port", new_value);
    }

    public Font getMenuFont() {
        return getFont("editing", "menu", "font");
    }

    public Font getGeneralFont() {
        return getFont("editing", "general", "font");
    }

    public void setGeneralFont(String new_name, int new_style, int new_size) {
        setFont("editing", "general", "font", new_name, new_style, new_size);
    }

    public Color getGeneralColor() {
        return getColor("editing", "general", "color");
    }

    public void setGeneralColor(int new_r, int new_g, int new_b) {
        setColor("editing", "general", "color", new_r, new_g, new_b);
    }

    public Color getGeneralBgColor() {
        return getColor("editing", "general", "backgroundcolor");
    }

    public void setGeneralBgColor(int new_r, int new_g, int new_b) {
        setColor("editing", "general", "backgroundcolor", new_r, new_g, new_b);
    }

    public Font getCommentFont() {
        return getFont("editing", "comment", "font");
    }

    public void setCommentFont(String new_name, int new_style, int new_size) {
        setFont("editing", "comment", "font", new_name, new_style, new_size);
    }

    public Color getCommentColor() {
        return getColor("editing", "comment", "color");
    }

    public void setCommentColor(int new_r, int new_g, int new_b) {
        setColor("editing", "comment", "color", new_r, new_g, new_b);
    }

    public Font getKeywordFont() {
        return getFont("editing", "keyword", "font");
    }

    public void setKeywordFont(String new_name, int new_style, int new_size) {
        setFont("editing", "keyword", "font", new_name, new_style, new_size);
    }

    public Color getKeywordColor() {
        return getColor("editing", "keyword", "color");
    }

    public void setKeywordColor(int new_r, int new_g, int new_b) {
        setColor("editing", "keyword", "color", new_r, new_g, new_b);
    }

    public Font getStringFont() {
        return getFont("editing", "string", "font");
    }

    public void setStringFont(String new_name, int new_style, int new_size) {
        setFont("editing", "string", "font", new_name, new_style, new_size);
    }

    public Color getStringColor() {
        return getColor("editing", "string", "color");
    }

    public void setStringColor(int new_r, int new_g, int new_b) {
        setColor("editing", "string", "color", new_r, new_g, new_b);
    }

    public Font getNumericFont() {
        return getFont("editing", "numeric", "font");
    }

    public void setNumericFont(String new_name, int new_style, int new_size) {
        setFont("editing", "numeric", "font", new_name, new_style, new_size);
    }

    public Color getNumericColor() {
        return getColor("editing", "numeric", "color");
    }

    public void setNumericColor(int new_r, int new_g, int new_b) {
        setColor("editing", "numeric", "color", new_r, new_g, new_b);
    }

    public Font getCharacterFont() {
        return getFont("editing", "character", "font");
    }

    public void setCharacterFont(String new_name, int new_style, int new_size) {
        setFont("editing", "character", "font", new_name, new_style, new_size);
    }

    public Color getCharacterColor() {
        return getColor("editing", "character", "color");
    }

    public void setCharacterColor(int new_r, int new_g, int new_b) {
        setColor("editing", "character", "color", new_r, new_g, new_b);
    }

    public String getShortcut(String key) {
        return getSCKeyAndModifiers("Shortcuts", key);
    }

    public void setShortcut(String key, String new_SCkey, String new_modifiers) {
        setSCKeyAndModifiers("Shortcuts", key, new_SCkey, new_modifiers);
    }

    public void save(String xmlpath) {
        saveXML(xmlpath, document, e);
    }

    private boolean getMode(String node, String parent, String mode) {
        String value = null;
        NodeList nl = e.getElementsByTagName(node);
        Element elem = (Element) nl.item(0);
        if (parent.equals("")) {
            value = elem.getAttribute(mode);
        }
        else {
            Element subnode = getElement(node, parent, "");
            value = subnode.getAttribute(mode);
        }
        if (value.equals("true")) {
            return true;
        }
        else {
            return false;
        }
    }

    private void setMode(String node, String parent, String mode, boolean new_mode) {
        Element elem = null;
        if (parent.equals("")) {
            NodeList nl = e.getElementsByTagName(node);
            elem = (Element) nl.item(0);
        }
        else {
            elem = getElement(node, parent, "");
        }
        elem.removeAttribute(mode);
        if (new_mode) {
            elem.setAttribute(mode, "true");
        }
        else {
            elem.setAttribute(mode, "false");
        }
    }

    private String getFirstChildValue(String node, String child) {
        String value = null;
        NodeList nl = e.getElementsByTagName(node);
        Element elem = (Element) nl.item(0);
        value = elem.getAttribute(child);
        return value;
    }

    private void setFirstChildValue(String node, String child, String new_value) {
        NodeList nl = e.getElementsByTagName(node);
        Element elem = (Element) nl.item(0);
        elem.removeAttribute(child);
        elem.setAttribute(child, new_value);
    }

    private String getSecondChildValue(String node, String parent, String key) {
        String value = null;
        Element elem = getElement(node, parent, "");
        value = elem.getAttribute(key);
        return value;
    }

    private void setSecondChildValue(String node, String parent, String key, String new_value) {
        Element elem = getElement(node, parent, "");
        elem.removeAttribute(key);
        elem.setAttribute(key, new_value);
    }

    private Color getColor(String node, String subnode, String type) {
        int r = 0;
        int g = 0;
        int b = 0;
        Element elem = getElement(node, subnode, type);        
        r = Integer.valueOf(elem.getAttribute("r")).intValue();
        g = Integer.valueOf(elem.getAttribute("g")).intValue();
        b = Integer.valueOf(elem.getAttribute("b")).intValue();
        
        return new Color(r, g, b);
    }

    private void setColor(String node, String subnode, String type, 
                                 int new_r, int new_g, int new_b) {
        Element elem = getElement(node, subnode, type); 
        elem.removeAttribute("r");
        elem.setAttribute("r", ""+new_r);
        elem.removeAttribute("g");
        elem.setAttribute("g", ""+new_g);
        elem.removeAttribute("b");
        elem.setAttribute("b", ""+new_b);
    }

    private Font getFont(String node, String subnode, String type) {
        String name = null;
        String Str_style = null;
        int style = 0;
        int size = 0;
        Element elem = getElement(node, subnode, type);
        name = elem.getAttribute("name");
        Str_style = elem.getAttribute("style");
            if (Str_style.trim().toLowerCase().equals("bold")) {
                style = Font.BOLD;
            }
            else if (Str_style.trim().toLowerCase().equals("italic")) {
                style = Font.ITALIC;
            }
            else if (Str_style.trim().toLowerCase().equals("bold, italic")) {
                style = Font.BOLD|Font.ITALIC;
            }
            else {
                style = Font.PLAIN;
            }
        size = Integer.valueOf(elem.getAttribute("size")).intValue();

        return new Font(name, style, size);
    }

    private void setFont(String node, String subnode, String type, 
                                String new_name, int new_style, int new_size) {
        Element elem = getElement(node, subnode, type);
        elem.removeAttribute("name");
        elem.setAttribute("name", new_name);
        elem.removeAttribute("style");
            if (new_style == Font.BOLD) { 
                elem.setAttribute("style", "bold");
            }
            else if (new_style == Font.ITALIC) { 
                elem.setAttribute("style", "italic");
            }
            else if (new_style == Font.PLAIN) {
                elem.setAttribute("style", "plain");
            }
        elem.removeAttribute("size");
        elem.setAttribute("size", ""+new_size);
    }

    private void setSCKeyAndModifiers(String node, String subnode, String new_value, String new_modifiers) {
        Element elem = getElement(node, subnode, "");
        elem.removeAttribute("SCkey");
        elem.setAttribute("SCkey", new_value);
        elem.removeAttribute("modifiers");
        elem.setAttribute("modifiers", new_modifiers);
    }

    private String getSCKeyAndModifiers(String node, String subnode) {
        try {
            Element elem = getElement(node, subnode, "");
            String key = elem.getAttribute("SCkey");
            String modifiers = elem.getAttribute("modifiers");
            if (key.equals("") || modifiers.equals("")) { 
                return "+&+";
            }
            else {
                return (key + "+&+" + modifiers);
            }
        } catch (Exception ex) {
            return "!";
        }
    }

    private Element getElement(String main, String subnode, String type) {
        NodeList main_nl = e.getElementsByTagName(main);
        Element main_elem = (Element) main_nl.item(0);
        NodeList sub_nl = main_elem.getElementsByTagName(subnode);
        Element sub_elem = (Element) sub_nl.item(0);
        if (!type.equals("")) {
            NodeList attr_nl = sub_elem.getElementsByTagName(type);
            Element attr_elem = (Element) attr_nl.item(0);
            return attr_elem;
        }
        else {
            return sub_elem;
        }
    }

    protected static void saveXML(String path, Document doc, Element root) {
        try {
            PrintWriter out = new PrintWriter(new OutputStreamWriter (new FileOutputStream(path)));
            org.apache.xml.serialize.OutputFormat outForm = 
                new org.apache.xml.serialize.OutputFormat(doc);
            org.apache.xml.serialize.XMLSerializer serializer = 
                new org.apache.xml.serialize.XMLSerializer(out, outForm);
            outForm.setIndenting(true);
            serializer.serialize(root);
        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }
}

